// hashtab.cpp
// 6/7/99
//	Hash table demonstration
// Ken Brownsey

#include <iostream.h>
#include "hashtab.h"

typedef Sequence_T Bucket_T;

typedef struct {
	Bucket_T	*Buckets;
	BOOLEAN		Q_Empty;
	int			NumLocations;
	int         NumUsedLocations;
}	HashTableRec_T, *HashTableRec_Ptr_T;



//	PRE	NumLocations > 0
// POST	HashTable' contains NumLocations
//			buckets
//			AND Q_EmptyHashTable(HashTable')
void
Create_HashTable(HashTable_T &HashTable, int NumLocations) {
	HashTableRec_Ptr_T	NewHT;
	int	Index;

	NewHT = new HashTableRec_T;
	NewHT->Buckets = new Bucket_T[NumLocations];
	for (Index = 0; Index < NumLocations; Index++) {
		NewHT->Buckets[Index] = Create_Sequence();
	}
	NewHT->NumUsedLocations = 0;
	NewHT->NumLocations = NumLocations;
	HashTable = (void *) NewHT;
}


//	PRE
//	POST	IF HashTable is empty
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_EmptyHashTable(HashTable_T HashTable) {
	HashTableRec_Ptr_T	HT;

	HT = (HashTableRec_Ptr_T) HashTable;

	return(HT->NumUsedLocations == 0);
}


//	PRE	TRUE
//	POST	Key is hashed to a value in range [0..NumLocations-1]
int
Hash_Key(Key_T	Key, int NumLocations) {

	return(Key % NumLocations);
}


//	PRE	TRUE
// POST	HashTable' = HashTable with Value added
void
Add_Value(int Value, HashTable_T HashTable)  {
	HashTableRec_Ptr_T	HT;
	int	Index;

	HT = (HashTableRec_Ptr_T) HashTable;

	Index = Hash_Key(Value, HT->NumLocations);
	if (Q_Empty(HT->Buckets[Index])) {
		HT->NumUsedLocations++;
		HT->Q_Empty = FALSE;
	}
	Append_After_Current(Value, HT->Buckets[Index]);
}


//	PRE	TRUE
// POST	IF Value is in HashTable
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_PresentValue(int Value, HashTable_T HashTable) {
	HashTableRec_Ptr_T	HT;
	int	Index;
	Bucket_T	Bucket;

	HT = (HashTableRec_Ptr_T) HashTable;

	Index = Hash_Key(Value, HT->NumLocations);
	Bucket = HT->Buckets[Index];
	if (!Q_Empty(Bucket)) {
		for(	Go_To_Head(Bucket);
				(Value != Get_Current(Bucket)) && (!Q_At_End(Bucket));
				Go_To_Next(Bucket)){
		}
		return(Value == Get_Current(Bucket));
	}
	else {
		return(FALSE);
	}
}


//	PRE	TRUE
//	POST 	Q_EmptyHashTable(HashTable)
void
Make_Empty(HashTable_T HashTable)  {
	HashTableRec_Ptr_T	HT;
	int	Index;
	Bucket_T	Bucket;

	HT = (HashTableRec_Ptr_T) HashTable;

	for (Index = 0; Index < HT->NumLocations; Index++) {
		Bucket = HT->Buckets[Index];
		for (;!Q_Empty(Bucket);) {
			Delete_Current(Bucket);
		}
	}
	HT->Q_Empty = TRUE;
	HT->NumUsedLocations = 0;
}


//	PRE	TRUE
//	POST	All memory allocted to HashTable is freed up
void
Delete_HashTable(HashTable_T &HashTable)  {
	HashTableRec_Ptr_T	HT;

	Make_Empty(HashTable);
	HT = (HashTableRec_Ptr_T) HashTable;

	delete [] HT->Buckets;
	HashTable = NULL;
}


//	PRE	TRUE
//	POST	RETURNS size of Bucket
int
Get_BucketSize(Bucket_T Bucket) {
	int	Count = 0;

	if (!Q_Empty(Bucket)) {
		Go_To_Head(Bucket);
		for(;!Q_At_End(Bucket);) {
			Count++;
			Go_To_Next(Bucket);
		}
		Count++;
	}
	return(Count);
}


//	PRE	TRUE
//	POST	Prints out contents of Bucket
void
Print_Bucket(Bucket_T Bucket) {

	if (!Q_Empty(Bucket)) {
		Go_To_Head(Bucket);
		for(;!Q_At_End(Bucket);) {
			cout	<<	Get_Current(Bucket)
					<<	", ";
			Go_To_Next(Bucket);
		}
		cout	<<	Get_Current(Bucket)
				<<	".\n";
	}
}


//	PRE	TRUE
//	POST	Prints out contents of HashTable bucket by bucket
void
Print_HashTable(HashTable_T &HashTable)  {
	HashTableRec_Ptr_T	HT;
	int	Index;

	HT = (HashTableRec_Ptr_T) HashTable;
	for(Index = 0; Index < HT->NumLocations; Index++) {
		cout	<<	"\n\nBucket "
				<<	Index
				<<	" contains "
				<<	Get_BucketSize(HT->Buckets[Index])
				<<	" values as follows\n";
		Print_Bucket(HT->Buckets[Index]);
	}
}

