// searchdb.cpp
// 6/7/99
// Set up unordered array
// Ken Brownsey


#include <iostream.h>
#include "sortdb.h"
#include <stdlib.h>



//	PRE	Size > 0
//	POST	IF Value present in elements [0..TopLimit] of Array
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE

BOOLEAN
Q_Present(int	Value, SortArray_T Array, int TopLimit) {
	int Index;

	for(Index = 0; Index <= TopLimit; Index++) {
		if (Value == Array[Index]) {
			return(TRUE);
		}
	}
	return(FALSE);
}


//	PRE	Size > 0
//	POST	Elements [0..Size - 1] of Array
//			populated with values 1<=Val<=MaxVal
void
Populate_Random_Array(SortArray_T Array, int Size, int MaxVal) {
	int	Index;
	int	NextVal;

	Array[0] = 1 + rand() % MaxVal;
	for(Index = 1; Index < Size; Index++) {
		NextVal = 1 + rand() % MaxVal;
		for(;Q_Present(NextVal, Array, Index - 1);) {
			NextVal = 1 + rand() % MaxVal;
		}
		Array[Index] = NextVal;
	}
}


//	PRE	Num > 0
//	POST	RETURNS number of digits in Num
int
Find_NumDigits(int Num) {
	int	Index;

	for(Index = 0; Num > 0; Index++) {
		Num = Num / 10;
	}
	return(Index);
}


//	PRE	Hi >= Lo
//	POST	All values in positions [Lo..Hi]
//			are printed to screen
//			IF DistPosition >= 0
//				the element in DistPosition is highlighted
//				with a >
void
Print_Array_Interval(SortArray_T Array, int Lo, int Hi, int DistPosition) {
	#define FieldWidth 4
	int	Index;
	int	SpaceIndex;
	int	HighLightCorrection = 0;

	for(Index = 0; Index <= Hi; Index++) {
		if (Index < Lo) {
			for (SpaceIndex = 1; SpaceIndex <= FieldWidth; SpaceIndex++) {
				cout	<<	" ";
			}
		}
		else {
			if (Index == DistPosition) {
				HighLightCorrection = 1;
			}
			for (SpaceIndex = 1; SpaceIndex <= FieldWidth - Find_NumDigits(Array[Index]) - HighLightCorrection; SpaceIndex++) {
				cout	<<	" ";
			}
			if (Index == DistPosition) {
				HighLightCorrection = 0;
				cout	<< ">";
			}
			cout	<<	Array[Index];
		}
	}
}


//	PRE	Hi >= Lo
//	POST	All values in Array
//			are printed to screen
//			AND element at Lo is highlighted with a L
//			AND element at Hi is highlighted with a H
//			AND IF DistPosition >= 0
//					the element at DistPosition is highlighted
//					with a >
void
Print_HighLightedArray(SortArray_T Array, int Lo, int Hi, int MarkedPosition, char Mark, int Size) {
	#define FieldWidth 4
	int	Index;
	int	SpaceIndex;
	int	HighLightCorrection = 0;

	for(Index = 0; Index < Size; Index++) {
		if (	((Index == Lo) && (Lo >= 0)) ||
				((Index == MarkedPosition) && (MarkedPosition >= 0)) ||
				((Index == Hi) && (Hi >= 0))) {
			HighLightCorrection = 1;
		}
		for (SpaceIndex = 1; SpaceIndex <= FieldWidth - Find_NumDigits(Array[Index]) - HighLightCorrection; SpaceIndex++) {
				cout	<<	" ";
		}
		if ((Index == Lo) && (Lo >= 0)) {
			HighLightCorrection = 0;
			cout	<< "L";
		}
		else {
			if ((Index == Hi) && (Hi >= 0)) {
				HighLightCorrection = 0;
				cout	<< "H";
			}
			else {
				if ((Index == MarkedPosition) && (MarkedPosition >= 0)) {
					HighLightCorrection = 0;
					cout	<< Mark;
				}
			}

		}
		cout	<<	Array[Index];
	}
}


//	PRE	MaxVal >= Size > 0
// POST	Array' is an unsorted array of size Size
//			unique elements in range 1..MaxVal
void
Make_UnOrdered_RandomArray(SortArray_T &Array, int	Size, int MaxVal) {

	if (Array != NULL) {
		delete [] Array;
	}
	Array = new int[Size];
	Populate_Random_Array(Array, Size, MaxVal);
}


//	PRE	Size > 0
//	POST	Array sorted into ascending order
void
Bubble_Sort(SortArray_T Array, int Size) {
	int	LoSorted,
			Index;
	int	Temp;
	BOOLEAN	Q_Sorted;

	for(Q_Sorted = FALSE, LoSorted = Size; !Q_Sorted; LoSorted--) {
		Q_Sorted = TRUE;
		for(Index = 0; Index < LoSorted - 1; Index++) {
/*char Dummy;
Print_Array_Interval(Array, 0, Size - 1);
cin	>> Dummy;
cout	<<	"\n";*/
			if (Array[Index] > Array[Index + 1]) {
				Temp = Array[Index];
				Array[Index] = Array[Index + 1];
				Array[Index + 1] = Temp;
				Q_Sorted = FALSE;
			}
		}
	}
}


//	PRE	MaxVal >= Size > 0
// POST	Array' is a sorted array of size Size
//			unique elements in range 1..MaxVal
void
Make_Ordered_RandomArray(SortArray_T &Array, int	Size, int MaxVal) {

	if (Array != NULL) {
		delete [] Array;
	}
	Array = new int[Size];
	Populate_Random_Array(Array, Size, MaxVal);
	Bubble_Sort(Array, Size);
}


//	PRE	Size > 0
//	POST	Array sorted into DESCENDING order
void
ReverseBubble_Sort(SortArray_T Array, int Size) {
	int	LoSorted,
			Index;
	int	Temp;
	BOOLEAN	Q_Sorted;

	for(Q_Sorted = FALSE, LoSorted = Size; !Q_Sorted; LoSorted--) {
		Q_Sorted = TRUE;
		for(Index = 0; Index < LoSorted - 1; Index++) {
/*char Dummy;
Print_Array_Interval(Array, 0, Size - 1);
cin	>> Dummy;
cout	<<	"\n";*/
			if (Array[Index] < Array[Index + 1]) {
				Temp = Array[Index];
				Array[Index] = Array[Index + 1];
				Array[Index + 1] = Temp;
				Q_Sorted = FALSE;
			}
		}
	}
}


//	PRE	MaxVal >= Size > 0
// POST	Array' is a reverse sorted array of size Size
//			unique elements in range 1..MaxVal
void
Make_ReverseOrdered_RandomArray(SortArray_T &Array, int	Size, int MaxVal) {

	if (Array != NULL) {
		delete [] Array;
	}
	Array = new int[Size];
	Populate_Random_Array(Array, Size, MaxVal);
	ReverseBubble_Sort(Array, Size);
}




