// srchdemo.cpp
// 6/7/99
// Demo of searching ordered and unordered sequence
// Ken Brownsey

#include <iostream.h>
#include "searchdb.h"
#include <stdlib.h>
#include <math.h>

BOOLEAN	Q_Ordered;
int		MaxVal;
int		LowFrequenceyFactor;

//	PRE	TRUE
//	POST	IF ((Target > MaxVal/n) && (Target < (n - 1)*(MaxVal/n)))
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_LowFrequencey(int Target, int n) {
	return((Target > ((float) MaxVal)/n) && (Target < (n - 1)*((float)MaxVal/n)));
}


//	PRE	TRUE
//	POST	IF DB is not empty all values are printed to screen
//			ELSE message to say that DB is empty
void
Print_All_Values(SearchDB_T DB, int LowFrequenceyFactor) {
	int	NextVal;
	int	SpaceIndex;
	int	LowFrequenceyCount;
	int	LowFrequenceyCountInTop50Percent;
	int	Index;
	int	HalfWayIndex;
	BOOLEAN	Q_Finished;
	#define FieldWidth 5

	if (!Q_Empty(DB)) {
		cout	<< "\n******************\nValues of the table are as follows\n\n";
		Index = 1;
		HalfWayIndex = Get_Size(DB)/2;
		LowFrequenceyCountInTop50Percent = 0;
		LowFrequenceyCount = 0;
		Q_Finished = FALSE;
		Go_To_Head(DB);
		for(;!Q_Finished;) {
			NextVal = Get_Current(DB);
			if (Q_At_End(DB)) {
				Q_Finished = TRUE;
			}
			if (!Q_LowFrequencey(NextVal, LowFrequenceyFactor)) {
				cout	<< "<";
			}
			else {
				LowFrequenceyCount++;
				if (Index <= HalfWayIndex) {
					LowFrequenceyCountInTop50Percent++;
				}
				cout	<< " ";
			}
			cout	<<	NextVal;
			for (SpaceIndex = 1; SpaceIndex <= FieldWidth - Find_NumDigits(NextVal) - 1; SpaceIndex++) {
				cout	<<	" ";
			}
			Go_To_Next(DB);
			Index++;
		}

		for (SpaceIndex = 1; SpaceIndex <= FieldWidth - Find_NumDigits(NextVal) - 1; SpaceIndex++) {
			cout	<<	" ";
		}
		cout	<< "\n******************\nThat's all the values!\n";
		cout	<<	LowFrequenceyCount
				<<	" are low frequency queries \n";
		cout	<<	LowFrequenceyCountInTop50Percent
				<<	" of first "
				<< HalfWayIndex
				<<	" are low frequency queries \n";
	}
	else {
		cout	<< ("\nDatabase is empty - no keys to print\n\n");
	}
}


// PRE	TRUE
// POST	IF Value is in DB
//				Position is position of Value in DB
//				Value is first in DB'
//				RETURNS TRUE
//			ELSE
//				Position' is that of last in DB
//				AND	Q_At_End(DB')
//				RETURNS FALSE
BOOLEAN
Q_Heuristic_Present(int Value, SearchDB_T DB, int &Position) {

	Go_To_Head(DB);
	for (Position = 1; (!Q_At_End(DB)) && (Value != Get_Current(DB));Position++) {
		Go_To_Next(DB);
	}
	// EITHER 	Q_At_End AND Value != Get_Current(DB)
	// OR			Q_At_End AND Value == Get_Current(DB)
	// OR			NOT Q_At_End AND Q_At_End AND Value == Get_Current(DB)
	if (Value == Get_Current(DB)) {
		Delete_Current(DB);
//		The current element is the the first
//		- post-condition of Delete_Current - if NOT Q_Empty(DB')
		Insert_Before_Current(Value, DB);
		return(TRUE);
	}
	else {
		return(FALSE);
	}
}


//	PRE	TRUE
//	POST	User is prompted for Size of OrdDB
//			IF Size > 0
//				OrdDB contains Size elements in order
//			ELSE
//				message
void
Build_UnOrdered_RandomDB(SearchDB_T UnOrdRandDB, int &MaxVal) {
	int	Size;

	Make_Empty(UnOrdRandDB);
	cout	<< "\nPlease type in number of elements ==> ";
	cin	>>	Size;
	if (Size > 0 ) {
		Make_UnOrdered_RandomDB(UnOrdRandDB, Size, MaxVal);
		Q_Ordered = FALSE;
	}
	else {
		cout	<<	"\n"
				<<	Size
				<<	" is less than 0";
	}
}


//	PRE	TRUE
//	POST	User is prompted for value
//			IF value > 0
//				this becomes new value of LowFrequenceyFactor
//			ELSE
//				message
void
Reset_LowFrequenceyFactor(int &LowFrequenceyFactor) {
	int	InputRF;

	cout	<< "\nPlease type in LowFrequenceyFactor > 2 ==> ";
	cin	>>	InputRF;
	if (InputRF > 0 ) {
		LowFrequenceyFactor = InputRF;
	}
	else {
		cout	<<	"\n"
				<<	InputRF
				<<	" is less than 3";
	}
}


//	PRE	TRUE
//	POST	User is prompted for
//			maximum possible value MaxVal
//			AND number of searches NumSearches
//			IF MaxVal > 0
//				IF NumSearches > 0
//					MaxVal searches are done on DB
//					AND results displayed
//				ELSE
//					message
//			ELSE
//				message

void
Do_Multiple_Random_Searches(SearchDB_T SearchDB, int MaxVal, int LowFrequenceyFactor) {
	float	NumSearches;
	float	Index;
	int	Target;
	int	Position;
	float	NumComps;
	float	NumHits;
	float	LowFrequenceyValueCount;

	if (MaxVal >0) {
		cout	<<	"\nPlease input number of searches to be done ==> ";
		cin	>> NumSearches;
		if (NumSearches > 0) {
			NumComps = 0;
			NumHits = 0;
			LowFrequenceyValueCount = 0;
			for(Index = 1; Index <= NumSearches; Index++) {
				Target = 1 + rand() % MaxVal;
				for (;(Q_LowFrequencey(Target, LowFrequenceyFactor))&&(rand() % 8 < 7);) {
					Target = 1 + rand() % MaxVal;
				}
				if (Q_LowFrequencey(Target, LowFrequenceyFactor)) {
					LowFrequenceyValueCount++;
				}
				if (Q_Heuristic_Present(Target, SearchDB, Position)) {
						NumHits++;
				}
				NumComps = NumComps + Position;
			}

			cout	<<	"\nThere were "
					<<	NumComps
					<<	" comparisons"
					<<	"\nThere were "
					<<	NumHits
					<<	" hits"
					<<	"\nAverage of  "
					<<	NumComps/NumSearches
					<<	"\n"
					<< LowFrequenceyValueCount
					<<	" targets were from low frequency values"
					<<	" - average of "
					<<	LowFrequenceyValueCount/NumSearches;
		}
	}
}


// PRE  -	TRUE
// POST -	menu displayed
void
Menu() {
	cout	<<	"***	put new record into (k)ey order **********\n"
			<< "***	(p)rint all values        ****************\n"
			<< "***	build (u)nordered random database     ****\n"
			<< "***	do multiple random heuristic (s)earches **\n"
			<< "***	(r)eset low frequencey factor           **\n"
			<< "***	make database (e)mpty  *******************\n"
			<< "***   e(x)it					      ***************\n"
			<< "***   (m)enu						   ***************\n";
}


// PRE	TRUE
// POST	Clears up
void
Initialise(SearchDB_T &DB) {

	DB = Create_Sequence();
	LowFrequenceyFactor = 8;
	cout	<< "Here we go !\n";
}


// PRE	TRUE
// POST Goes into a prompt, read and execute cycle
//		until 'x' is the character read
void
Loop(SearchDB_T DB) {

	char	Request;
	int	RequestCount;

	for (Request = 'm', RequestCount = 0; Request != 'x'; RequestCount++) {

		switch(Request) {

			case 'p' :	Print_All_Values(DB, LowFrequenceyFactor);
				break;
			case 'u' :	Build_UnOrdered_RandomDB(DB, MaxVal);
				break;
			case 's' :	Do_Multiple_Random_Searches(DB, MaxVal, LowFrequenceyFactor);
				break;
			case 'r' :	Reset_LowFrequenceyFactor(LowFrequenceyFactor);
				break;
			case 'e' :	Make_Empty(DB);
				break;
			case 'm' : Menu();
				break;
			default   :
				cout	<< "\n m for menu \n";
		}
		cout	<< "\n\nNext request ["
				<< RequestCount
				<< "] ==> ";
		cin	>> Request;
	}
}


// PRE	TRUE
// POST	Clears up
void
Finalise(SearchDB_T DB) {
	Make_Empty(DB);
	Free_Sequence(DB);
	cout	<< "That is it !\n";
}


void
main() {
	SearchDB_T	DataBase;

	Initialise(DataBase);
	Loop(DataBase);
	Finalise(DataBase);
}
