// Ken Brownsey
// 14/5/99
// vert_bst.h
// VertBST_T

#ifndef	VERT_BST_H
#define 	VERT_BST_H

#include "VStore.h"


typedef void *VertBST_T;
typedef void *VertBST_Iterator_T;

// PRE	TRUE
// POST	RETURNS the empty tree
VertBST_T
Create_VertTree();


// PRE	TRUE
// POST	IF Tree is empty
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_EmptyVertTree(VertBST_T Tree);


// PRE	Tree is not empty
// POST	RETURNS the record in the root
VertKAPRec_T
Vert_Get_Root(VertBST_T Tree);


// PRE	TRUE
// POST	IF Tree is not empty
//				RETURNS the left sub tree of Tree
//			ELSE
//				RETURNS the empty tree
VertBST_T
Vert_Get_Left(VertBST_T Tree);



// PRE	TRUE
// POST	IF Tree is not empty
//				RETURNS the right sub tree of Tree
//			ELSE
//				RETURNS the empty tree
VertBST_T
Vert_Get_Right(VertBST_T Tree);


// PRE	TRUE
// POST	RETURNS the tree whose head is Root and tail is Tree
VertBST_T
Make_VertTree(VertBST_T Left, VertKAPRec_T Element, VertBST_T Right);


// PRE	TRUE
// POST	if record with key VertKey is in VertBST
//				RETURNS TRUE
//			else
//				RETURNS FALSE
BOOLEAN
Q_VertBST_Present_R(int (*Get_Key) (VertKAPRec_T), int VertKey, VertBST_T VertBST);


// PRE	NOT Q_VertBST_Present_R(Rec, VertBST)
// POST	RETURNS VertBST with Rec inserted
void
VertBST_Insert_R(int (*Get_Key) (VertKAPRec_T), VertKAPRec_T Rec, VertBST_T &VertBST);


// PRE	Q_VertBST_Present_R(VertKey, VertBST)
// POST	RETURNS record with key value VertKey from VertBST
VertKAPRec_T
VertBST_Retrieve_R(int (*Get_Key) (VertKAPRec_T), int VertKey, VertBST_T VertBST);


// PRE	TRUE
// POST	if (Q_VertBST_Present(Rec, VertBST)
//				RETURNS VertBST with Rec deleted
//			else
//				RETURNS VertBST
void
VertBST_Delete_R(int (*Get_Key) (VertKAPRec_T), int VertKey, VertBST_T &VertBST);


// PRE	VertBST is primary ordered by Get_PKey()
// POST	if VertBST contains Rec such that
//				Get_PKey(Rec) = PKey
//				AND Get_SKey(Rec) = SKey
//				RETURNS VertBST with Rec deleted
//			else
//				RETURNS VertBST
void
VertTwoKeysBST_Delete_R(	int (*Get_PKey) (VertKAPRec_T), int (*Get_SKey) (VertKAPRec_T),
						int PVertKey,  int SVertKey, VertBST_T &VertBST);


// PRE	TRUE
// POST	RETURNS (physically distinct) copy of Tree
//						i.e. same structure
VertBST_T
Copy_VertTree(int (*Get_Key) (VertKAPRec_T), VertBST_T VertBST);


// PRE	TRUE
// POST	IF VertBST contains records with key Key
//				RETURNS tree containing first occurrence as root
//			ELSE
//				RETURNS the empty tree
VertBST_T
Find_FirstVert_R(int (*Get_Key) (VertKAPRec_T), int Key, VertBST_T VertBST);


// PRE	TRUE
// POST	IF record with from key Key
//			is in VertBST apart from the root
//				(different from Find_First_R)
//				RETURNS tree with that record as root
//			ELSE
//				RETURNS the empty tree
VertBST_T
Find_NextDuplicateVert(int (*Get_Key) (VertKAPRec_T), int Key, VertBST_T VertBST);


// PRE	Q_VertBST_Present_R(Get_VertKAPRecKey(VertKAPRecUpdate), VertBST)
// POST	record with key value VertKey from VertBST is replaced by VertKAPRecUpdate
void
Update_Vert(VertKAPRec_T VertKAPRecUpdate, VertBST_T VertBST);


// PRE	TRUE
// POST	All memory allocated to KAPRec's in VertBST is freed
void
VertBST_Dispose_All_KAPRecs_R(VertBST_T &VertBST);


// PRE	TRUE
// POST	All memory for nodes allocated to VertBST is freed
//			AND Q_EmptyVertTree(VertBST)
// NOTES	This will not dispose of memory referenced by the elements
//			e.g. through KAPRec_T's
void
VertBST_Delete_All(VertBST_T &VertBST);


// PRE	TRUE
// POST	Iterator is created
VertBST_Iterator_T
Create_VertIterator();


// PRE	TRUE
// POST	IF Start_Iteration(Iterator, VertBST) has been executed
//				correctly for some value VertBST
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_Bound_To_VertBST(VertBST_Iterator_T Iterator);


// PRE	TRUE
// POST	IF iteration is at ordinally keyed first vert
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_StartOfVertIteration(VertBST_Iterator_T Iterator);


// PRE	Q_Bound_To_VertBST(Iterator)
// POST	IF iteration is at ordinally keyed last vert
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_EndOfVertIteration(VertBST_Iterator_T Iterator);


// PRE	NOT Q_EmptyTree(VertBST)
// POST	Iterator is reset with
//			ordinally first key vert as current vert
void
Start_VertIteration(VertBST_Iterator_T &Iterator, VertBST_T VertBST);

// PRE	Start_Iteration(&Iterator, VertBST) has been called
//			for some value VertBST
//			AND NOT Q_EndOfIteration(Iterator)
// POST	NOT Q_Iteration_Start(Iterator') AND
//					current is next node in iteration
void
VertIterate(VertBST_Iterator_T &Iterator);


// PRE     Q_Iteration_Started(Iterator)
// RETURNS current record in iteration
VertKAPRec_T
Get_CurrentVert(VertBST_Iterator_T Iterator);


// PRE	Q_Iteration_Started(Iterator)
//			AND Get_VertKAPRecKey(VertKAPRecUpdate) = Get_VertKAPRecKey(Get_CurrentVert(Iterator))
// POST	Get_CurrentVert(Iterator) = VertKAPRecUpdate))
//	NOTES	PRE (ii) required to maintain key ordering
void
Update_CurrentVert(VertKAPRec_T VertKAPRecUpdate, VertBST_Iterator_T Iterator);


// PRE	Q_Iteration_Started(Iterator)
// POST	RETURNS number of current record in iteration
int
Get_CurrentVertNumber(VertBST_Iterator_T Iterator);


// PRE	Create_Iterator(Iterator) has been called
//			AND since last, if any, call to
//			Destroy_Iterator(Iterator)
// POST	Iterator is destroyed
void
Destroy_VertIterator(VertBST_Iterator_T Iterator);


#endif