// Ken Brownsey
// 14/5/99
// arc_bst.h
// ArcBST_T

#ifndef	ARC_BST_H
#define 	ARC_BST_H

#include "AStore.h"


typedef void  *ArcBST_T;
typedef void *ArcBST_Iterator_T;

// PRE	TRUE
// POST	RETURNS the empty tree
ArcBST_T
Create_ArcTree();


// PRE	TRUE
// POST	IF Tree is empty
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_EmptyArcTree(ArcBST_T Tree);


// PRE	Tree is not empty
// POST	RETURNS the record in the root
ArcKAPRec_T
Arc_Get_Root(ArcBST_T Tree);


// PRE	TRUE
// POST	IF Tree is not empty
//				RETURNS the left sub tree of Tree
//			ELSE
//				RETURNS the empty tree
ArcBST_T
Arc_Get_Left(ArcBST_T Tree);



// PRE	TRUE
// POST	IF Tree is not empty
//				RETURNS the right sub tree of Tree
//			ELSE
//				RETURNS the empty tree
ArcBST_T
Arc_Get_Right(ArcBST_T Tree);


// PRE	TRUE
// POST	RETURNS the tree whose head is Root and tail is Tree
ArcBST_T
Make_ArcTree(ArcBST_T Left, ArcKAPRec_T Element, ArcBST_T Right);


// PRE	TRUE
// POST	IF Rec is in ArcBST
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_ArcBST_Present_R(int(*Get_Key) (ArcKAPRec_T), int ArcKey, ArcBST_T ArcBST);

// PRE	NOT Q_ArcBST_Present_R(Rec, ArcBST)
// POST	RETURNS ArcBST with Rec inserted
//void
//ArcBST_Insert_R(ArcKAPRec_T Rec, ArcBST_T &ArcBST);
void
ArcBST_Insert_R(int(*Get_Key) (ArcKAPRec_T), ArcKAPRec_T Rec, ArcBST_T &ArcBST);


// PRE	Q_ArcBST_Present_R(ArcKey, ArcBST)
// POST	returnS record with key value ArcKey from ArcBST
ArcKAPRec_T
ArcBST_Retrieve_R(int(*Get_Key) (ArcKAPRec_T), int ArcKey, ArcBST_T ArcBST);

// PRE	TRUE
// POST	if (Q_ArcBST_Present(Rec, ArcBST)
//				returnS ArcBST with Rec deleted
//			ELSE
//				returnS ArcBST
void
ArcBST_Delete_R(int(*Get_Key) (ArcKAPRec_T), int ArcKey, ArcBST_T &ArcBST);


// PRE	TRUE
// POST	All memory allocated to KAPRec's in ArcBST is freed
void
ArcBST_Dispose_All_KAPRecs_R(ArcBST_T &ArcBST);


// PRE	TRUE
// POST	All memeory allocated to ArcBST is freed
void
ArcBST_Delete_All(ArcBST_T &ArcBST);


// PRE	TRUE
// POST	Iterator is created
ArcBST_Iterator_T
Create_ArcIterator();


// PRE	TRUE
// POST	IF Start_Iteration(Iterator, ArcBST) has been executed
//				correctly for some value ArcBST
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
Q_Bound_To_ArcBST(ArcBST_Iterator_T Iterator);


// PRE	TRUE
// POST	IF iteration is at ordinally keyed first arc
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_StartOfArcIteration(ArcBST_Iterator_T Iterator);


// PRE	Q_Bound_To_ArcBST(Iterator)
// POST	IF iteration is at ordinally keyed last arc
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_EndOfArcIteration(ArcBST_Iterator_T Iterator);


// PRE	NOT Q_EmptyTree(ArcBST)
// POST	Iterator is reset with
//			ordinally first key arc as current arc
void
Start_ArcIteration(ArcBST_Iterator_T &Iterator, ArcBST_T ArcBST);

// PRE	Start_Iteration(&Iterator, ArcBST) has been called
//			for some value ArcBST
//			AND NOT Q_EndOfIteration(Iterator)
// POST	NOT Q_Iteration_Start(Iterator') AND
//					current is next node in iteration
void
ArcIterate(ArcBST_Iterator_T &Iterator);


// PRE     Q_Iteration_Started(Iterator)
// RETURNS current record in iteration
ArcKAPRec_T
Get_CurrentArc(ArcBST_Iterator_T Iterator);


// PRE	Q_Iteration_Started(Iterator)
// POST	RETURNS number of current record in iteration
int
Get_CurrentArcNumber(ArcBST_Iterator_T Iterator);


// PRE	Create_Iterator(Iterator) has been called
//			AND since last, if any, call to
//			Destroy_Iterator(Iterator)
// POST	Iterator is destroyed
void
Destroy_ArcIterator(ArcBST_Iterator_T Iterator);


// PRE	TRUE
// POST	IF ArcBST contains records with key VertKey
//				RETURNS tree containing first occurrence as root
//			ELSE
//				RETURNS NULL
ArcBST_T
Find_First_R(BOOLEAN(*Get_Key) (ArcKAPRec_T), VertKey_T VertKey, ArcBST_T ArcBST);


// PRE	TRUE
// POST	IF record with from key VertKey
//			is in ArcBST apart from the root
//				(different from Find_First_R)
//				RETURNS tree with that record as root
//			ELSE
//				RETURNS NULL
ArcBST_T
Find_NextDuplicate(BOOLEAN(*Get_Key) (ArcKAPRec_T), VertKey_T VertKey, ArcBST_T ArcBST);




#endif