#ifndef	GRAPH_H
#define 	GRAPH_H

#include "VertData.h"
#include "ArcData.h"
#include "VStore.h"
#include "AStore.h"

typedef	void 	*Graph_T;
typedef void *Traveller_T;


// **************************************************
// START CREATION AND BUILDING OPERATIONS
// **************************************************



// PRE	TRUE
// POST	RETURNS the empty graph
Graph_T
Create_Graph();


//	PRE	TRUE
// POST	Q_EmptyGraph(Graph')
//			AND all memory allocated to Graph is freed up
void
Make_Empty(Graph_T Graph);


// PRE	NOT Q_VertPresent(Get_VertKey(Vert), Graph)
// POST	Graph' = Graph UNION {Vert}
//			AND Q_FirstVert(Graph)
void
Add_Vert(VertDataRec_T Vert, Graph_T Graph);


// PRE	NOT Q_ArcPresent(Get_ArcKey(Arc), Graph)
//			AND Q_VertPresent(Get_FromVertKey(Arc), Graph)
//			AND Q_VertPresent(Get_ToVertKey(Arc), Graph)
// POST	Graph' = Graph UNION {Vert}
//			AND Q_FirstArc(Graph')
void
Add_Arc(ArcDataRec_T Arc, Graph_T Graph);



// **************************************************
// END CREATION AND BUILDING OPERATIONS
// **************************************************




// **************************************************
// START STATUS QUERY OPERATIONS
// **************************************************



// PRE	TRUE
// POST	IF Graph is empty
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_EmptyGraph(Graph_T Graph);


//	PRE	TRUE
//	POST	IF no arcs in Graph
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_NoArcs(Graph_T Graph);


//	PRE	TRUE
// POST	RETURNS number of vertices in Graph
int
Get_Size(Graph_T Graph);


//	PRE	TRUE
// POST	RETURNS number of arcs in Graph
int
Get_Number_Of_Arcs(Graph_T Graph);



// **************************************************
// END STATUS QUERY OPERATIONS
// **************************************************




// **************************************************
// START RETRIEVAL OPERATIONS
// **************************************************



// PRE	TRUE
// POST	IF vertex with key VertKey in Graph
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_VertPresent(VertKey_T	VertKey, Graph_T Graph);


//	PRE	Q_VertPresent(Vert, Graph)
// POST	RETURNS vertex with key VertKey from Graph
VertKAPRec_T
Retrieve_Vert(VertKey_T	VertKey, Graph_T Graph);


// PRE	TRUE
// POST	IF vertex with key ArcKey in Graph
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_ArcPresent(ArcKey_T	ArcKey, Graph_T Graph);


//	PRE	Q_ArcPresent(Arc, Graph)
// POST	RETURNS arc with key ArcKey from Graph
ArcKAPRec_T
Retrieve_Arc(ArcKey_T	ArcKey, Graph_T Graph);



// **************************************************
// END RETRIEVAL OPERATIONS
// **************************************************




// **************************************************
// START ITERATED TRAVERSAL OPERATIONS
// **************************************************



// PRE	NOT Q_EmptyGraph(Graph)
// POST	IF current element of Graph is the first
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_FirstVert(Graph_T Graph);


// PRE	NOT Q_EmptyGraph(Graph)
// POST	IF current element of Graph is the last
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_LastVert(Graph_T Graph);


// PRE	NOT Q_EmptyGraph(Graph)
// POST	current element of Graph is the first
void
Go_To_FirstVert(Graph_T Graph);


// PRE	NOT Q_EmptyGraph(Graph)
//			AND NOT Q_LastVert(Graph)
// POST	current is next vertex in iteration
void
Go_To_NextVert(Graph_T Graph);


// PRE	NOT Q_EmptyGraph(Graph)
// POST	RETURNS current vertex
VertKAPRec_T
Get_CurrentVertKAPRec(Graph_T Graph);


//	PRE	NOT Q_EmptyGraph(Graph)
//	POST	Q_CurrentVertMarked(Graph')
void
Set_CurrentMark(Graph_T	Graph);


//	PRE	NOT Q_EmptyGraph(Graph)
//	POST	NOT Q_CurrentVertMarked(Graph')
void
UnSet_CurrentMark(Graph_T	Graph);


// PRE	NOT Q_EmptyGraph(Graph)
// POST	IF current vertex is marked
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_CurrentVertMarked(Graph_T	Graph);


// PRE	NOT Q_NoArcs(Graph)
// POST	IF current arc of Graph is the first
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_FirstArc(Graph_T Graph);


// PRE	NOT Q_NoArcs(Graph)
// POST	IF current arc of Graph is the last
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_LastArc(Graph_T Graph);


// PRE	NOT Q_NoArcs(Graph)
// POST	Current arc of Graph is the first
void
Go_To_FirstArc(Graph_T Graph);


// PRE	NOT Q_NoArcs(Graph)
//			AND NOT Q_LastArc(Graph)
// POST	current is next arc in iteration
void
Go_To_NextArc(Graph_T Graph);


// PRE	NOT Q_NoArcs(Graph)
// POST	RETURNS current arc
ArcKAPRec_T
Get_CurrentArcKAPRec(Graph_T Graph);



// **************************************************
// END ITERATED TRAVERSAL OPERATIONS
// **************************************************




// **************************************************
// START TRAVELLER OPERATIONS
// **************************************************



// PRE	Q_VertPresent(VertKey, Graph)
// POST	Traveller_T is created, bound to Graph
//			AND set to vertex with key VertKey
//			AND RETURNed
//			IF NOT Q_Dead_End(Traveller')
//				Current arc of Traveller' is first
Traveller_T
Create_Traveller(VertKey_T	VertKey, Graph_T Graph);


//	PRE	TRUE
//	POST	RETURNS key of vertex where Traveller is at.
VertKey_T
Get_TravellerVertKey(Traveller_T Traveller);


// PRE	TRUE
// POST	IF VertKey is on Traveller's graph
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_Vert_For_Traveller(Traveller_T Traveller, VertKey_T	VertKey);


// PRE	Q_Vert_For_Traveller(Traveller, VertKey)
// POST	RETURNS VertKAPRec for VertKey
VertKAPRec_T
Get_Vert_From_Traveller(Traveller_T Traveller, VertKey_T	VertKey);


// PRE   Q_Vert_For_Traveller(Traveller, VertKey)
// POST	Traveller is reset to vertex with key VertKey
//			IF NOT Q_Dead_End(Traveller')
//				Current arc of Traveller' is first
void
Start_Traveller(Traveller_T Traveller, VertKey_T	VertKey);


// PRE   Q_Vert_For_Traveller(Traveller, VertKey)
//			AND VertKey is the key of the initial vertex of the
//				arc whose key is ArcKey
// POST	Traveller is reset to vertex with key VertKey
//			IF NOT Q_Dead_End(Traveller')
//			Current arc of Traveller' is that with key ArcKey
void
ReSet_Traveller(Traveller_T Traveller, VertKey_T	VertKey, ArcKey_T ArcKey);


//	PRE	TRUE
//	POST	IF Traveller is at the end of a path
//			i.e. there are no further arcs forward
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_Dead_End(Traveller_T Traveller);


//	PRE	NOT Q_Dead_End(Traveller)
//	POST	Current arc of Traveller' is the first
void
Go_To_FirstArc_From_Vertex(Traveller_T Traveller);


//	PRE	NOT Q_Dead_End(Traveller) AND NOT Q_No_More_Arcs(Traveller)
//	POST	Current arc of Traveller' is next one after that of Traveller
void
Go_To_NextArc_From_Vertex(Traveller_T Traveller);


//	PRE	TRUE
// POST	RETURNS VertKAPRec_T with Get_TravellerVertKey(Traveller)
VertKAPRec_T
Get_TravellerVertKAPRec(Traveller_T Traveller);


//	PRE	TRUE
//	POST	IF current arc in Traveller is last
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_No_More_Arcs(Traveller_T Traveller);


//	PRE	NOT Q_Dead_End(Traveller)
//	POST	Traveller' has moved along current arc of Traveller
//			AND IF NOT Q_Dead_End(Traveller')
//				current arc of Traveller' is the first
void
Move_To_Vertex_Along_Current_Arc(Traveller_T Traveller);


//	PRE	NOT Q_No_More_Arcs(Traveller)
//	POST	RETURNS Current arc for Traveller
ArcKAPRec_T
Get_CurrentTraveller_ArcKAPRec(Traveller_T Traveller);


//	PRE	TRUE
//	POST	Vertex for Traveller' is marked
void
Set_TravellerMark(Traveller_T Traveller);


//	PRE	TRUE
//	POST	Vertex for Traveller' is unmarked
void
UnSet_TravellerMark(Traveller_T Traveller);


//	PRE	TRUE
//	POST	IF Vertex with key VertKey is marked for Traveller
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_VertMarked_For_Traveller(VertKey_T VertKey, Traveller_T Traveller);


// PRE	Create_ArcIterator(Traveller) has been called
//			AND since last, if any, call to
//			Destroy_Iterator(Iterator)
// POST	Memory allocated to Traveller is freed
void
Destroy_Traveller(Traveller_T Traveller);



// **************************************************
// END TRAVELLER OPERATIONS
// **************************************************




// **************************************************
// START FILE OPERATIONS
// **************************************************



// PRE	TRUE
// POST	User is prompted for filename
//			IF this is for a pair of files containing valid vertices and arcs for a graph
//				Graph is emptied and then contents of files are loaded into Graph
//			ELSE
//				Q_EmptyGraph(Graph')
void
Load_Graph_From_File(Graph_T Graph);



// **************************************************
// END FILE OPERATIONS
// **************************************************

#endif
