// Ken Brownsey
// 14/5/99
// vert_bst.cpp
// VertBST_T directly pointer based implementation

#include "stack.h"
#include "my_const.h"
#include "vert_bst.h"

// record structure for iterator
typedef struct {
		Stack_T Stack;
		SElement_T Current;
		int CurrentNumber;
		BOOLEAN	Q_Bound;
}	VertBST_ItRec_T, *VertBST_ItRec_Ptr_T;

typedef struct TN_T {
	struct TN_T	*Left_Ptr,
					*Right_Ptr;
	VertKAPRec_T	Element;
} VertNode_T, *VertNode_Ptr_T;


// PRE	TRUE
// POST	RETURNS the empty tree
VertBST_T
Create_VertTree() {
	return(NULL);
}


// PRE	TRUE
// POST	IF Tree is empty
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_EmptyVertTree(VertBST_T Tree) {
	return(Tree == NULL);
}


// PRE  	NOT Q_EmptyVertTree(Tree)
// POST	RETURNS the root record
VertKAPRec_T
Vert_Get_Root(VertBST_T Tree) {
	VertNode_Ptr_T	Vert_Ptr;

	Vert_Ptr = (VertNode_Ptr_T) Tree;

	return(Vert_Ptr->Element);
}


// PRE	TRUE
// POST	IF NOT Q_EmptyVerttTree(Tree)
//				RETURNS the left sub tree of Tree
//			ELSE
//				RETURNS the empty tree
VertBST_T
Vert_Get_Left(VertBST_T Tree) {
	VertNode_Ptr_T	Vert_Ptr;

	Vert_Ptr = (VertNode_Ptr_T) Tree;

	if (Vert_Ptr == NULL) {
		return(NULL);
	}
	else {
		return((VertBST_T) (Vert_Ptr->Left_Ptr));
	}
}


// PRE	TRUE
// POST	IF NOT Q_EmptyVerttTree(Tree)
//				RETURNS the right sub tree of Tree
//			ELSE
//				RETURNS the empty tree
VertBST_T
Vert_Get_Right(VertBST_T Tree) {
	VertNode_Ptr_T	Vert_Ptr;

	Vert_Ptr = (VertNode_Ptr_T) Tree;

	if (Vert_Ptr == NULL) {
		return(NULL);
	}
	else {
		return((VertBST_T)  Vert_Ptr->Right_Ptr);
	}
}


// PRE	TRUE
// POST	RETURNS the tree whose root is Root,
//				left sub tree is Left,
//				right sub tree is Right
VertBST_T
Make_VertTree(VertBST_T Left, VertKAPRec_T Element, VertBST_T Right) {

	VertNode_Ptr_T	NewTree;

	NewTree = new VertNode_T;
	NewTree->Element = Element;
	NewTree->Left_Ptr = (VertNode_Ptr_T) Left;
	NewTree->Right_Ptr = (VertNode_Ptr_T) Right;
	return((VertBST_T)  NewTree);
}


// PRE	TRUE
// POST	IF record with key VertKey is in VertBST
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_VertBST_Present_R(VertKey_T VertKey, VertBST_T VertBST) {
	VertNode_Ptr_T	Vert_Ptr;

	Vert_Ptr = (VertNode_Ptr_T) VertBST;

	if (VertBST == NULL) {
		return(FALSE);
	}
	else {
		if (VertKey < Get_VertKAPRecKey(Vert_Ptr->Element)) {
			return(Q_VertBST_Present_R(VertKey, Vert_Ptr->Left_Ptr));
		}
		else {
			if (VertKey > Get_VertKAPRecKey(Vert_Get_Root(VertBST))) {
				return(Q_VertBST_Present_R(VertKey, Vert_Ptr->Right_Ptr));
			}
			else {
				return(TRUE);
			}
		}
	}
}


// PRE	NOT Q_VertBST_Present_R(Get_VertKAPRecKey(Rec), VertBST)
// POST	RETURNS VertBST with Rec inserted
void
VertBST_Insert_R(VertKAPRec_T Rec, VertBST_T &VertBST) {
	VertKey_T VertKey;
	VertNode_Ptr_T	Vert_Ptr,
					NewVert_Ptr,
					Parent;
	BOOLEAN	Q_OKInsert;

	Vert_Ptr = (VertNode_Ptr_T) VertBST;
	NewVert_Ptr = new(VertNode_T);
	NewVert_Ptr->Left_Ptr 	= NULL;
	NewVert_Ptr->Element 	= Rec;
	NewVert_Ptr->Right_Ptr 	= NULL;
	if (Vert_Ptr == NULL) {
		Vert_Ptr = NewVert_Ptr;
	}
	else {
		Parent = Vert_Ptr;
		Q_OKInsert = FALSE;
		VertKey = Get_VertKAPRecKey(Rec);
		for(;!Q_OKInsert;) {
			if (VertKey < Get_VertKAPRecKey(Parent->Element)) {
				if (Parent->Left_Ptr == NULL) {
					Parent->Left_Ptr = NewVert_Ptr;
					Q_OKInsert = TRUE;
				}
				else {
					Parent = Parent->Left_Ptr;
				}
			}
			else {
				if (Parent->Right_Ptr == NULL) {
					Parent->Right_Ptr = NewVert_Ptr;
					Q_OKInsert = TRUE;
				}
				else {
					Parent = Parent->Right_Ptr;
				}
			}
		}
	}
	VertBST = (VertBST_T)  Vert_Ptr;
}


// PRE	Q_VertBST_Present_R(VertKey, VertBST)
// POST	RETURNS record with key value VertKey from VertBST
VertKAPRec_T
VertBST_Retrieve_R(VertKey_T VertKey, VertBST_T VertBST) {
	VertNode_Ptr_T	Vert_Ptr;

	Vert_Ptr = (VertNode_Ptr_T) VertBST;

	if (VertKey < Get_VertKAPRecKey(Vert_Ptr->Element)) {
		return(VertBST_Retrieve_R(VertKey, Vert_Ptr->Left_Ptr));
	}
	else {
		if (VertKey > Get_VertKAPRecKey(Vert_Ptr->Element)) {
			return(VertBST_Retrieve_R(VertKey, Vert_Ptr->Right_Ptr));
		}
		else {
			return(Vert_Ptr->Element);
		}
	}
}


// PRE	Q_VertBST_Present_R(Get_VertKAPRecKey(VertKAPRecUpdate), VertBST)
// POST	record with key value VertKey from VertBST is replaced by VertKAPRecUpdate
void
Update_Vert(VertKAPRec_T VertKAPRecUpdate, VertBST_T VertBST) {

	VertNode_Ptr_T	Vert_Ptr;

	Vert_Ptr = (VertNode_Ptr_T) VertBST;

	if (Get_VertKAPRecKey(VertKAPRecUpdate) < Get_VertKAPRecKey(Vert_Ptr->Element)) {
		Update_Vert(VertKAPRecUpdate, (VertBST_T) Vert_Ptr->Left_Ptr);
	}
	else {
		if (Get_VertKAPRecKey(VertKAPRecUpdate) > Get_VertKAPRecKey(Vert_Ptr->Element)) {
			Update_Vert(VertKAPRecUpdate, (VertBST_T) (Vert_Ptr->Right_Ptr));
		}
		else {
			Vert_Ptr->Element = VertKAPRecUpdate;
		}
	}
}



// PRE	NOT Q_EmptyTree(VertBST)
// POST	RETURNS rightmost VertKAPRec_T of VertBST
VertKAPRec_T
Get_RightMostVert_R(	VertNode_Ptr_T	Vert_Ptr) {

	if (Vert_Ptr->Right_Ptr == NULL) {
		return(Vert_Ptr->Element);
	}
	else {
		return(Get_RightMostVert_R(Vert_Ptr->Right_Ptr));
	}
}


// PRE	TRUE
// POST	IF (Q_VertBST_Present(Rec, VertBST)
//				RETURNS VertBST with Rec deleted
//			ELSE
//				RETURNS VertBST
void
VertBST_Delete_R(VertKey_T VertKey, VertBST_T &VertBST) {
	VertNode_Ptr_T	Vert_Ptr;
	VertKAPRec_T NewRoot;

	Vert_Ptr = (VertNode_Ptr_T) VertBST;

	if (Vert_Ptr != NULL) {

		if (VertKey < Get_VertKAPRecKey(Vert_Ptr->Element)) {
			VertBST_Delete_R(VertKey, (VertBST_T)  Vert_Ptr->Left_Ptr);
		}

		else {
			if (VertKey > Get_VertKAPRecKey(Vert_Ptr->Element)) {
				VertBST_Delete_R(VertKey, (VertBST_T)  Vert_Ptr->Right_Ptr);
			}
			else {
				if (Vert_Ptr->Left_Ptr == NULL) {
					VertBST = (VertBST_T)  Vert_Ptr->Right_Ptr;
					delete(Vert_Ptr);
				}
				else {
					if (Vert_Ptr->Right_Ptr == NULL) {
						VertBST = (VertBST_T)  Vert_Ptr->Left_Ptr;
						delete(Vert_Ptr);
					}
					else {
						NewRoot = Get_RightMostVert_R(Vert_Ptr->Left_Ptr);
						Vert_Ptr->Element = NewRoot;
						VertBST_Delete_R(Get_VertKAPRecKey(NewRoot), (VertBST_T)  Vert_Ptr->Left_Ptr);
					}
				}
			}
		}
	}
}


// PRE	TRUE
// POST	All memory allocated to KAPRec's in VertBST is freed
void
VertBST_Dispose_All_KAPRecs_R(VertBST_T &VertBST) {
	VertNode_Ptr_T	Vert_Ptr;

	Vert_Ptr = (VertNode_Ptr_T) VertBST;

	if (Vert_Ptr != NULL) {
		VertBST_Dispose_All_KAPRecs_R((VertBST_T) Vert_Ptr->Left_Ptr);
		VertBST_Dispose_All_KAPRecs_R((VertBST_T) Vert_Ptr->Right_Ptr);
		Free_VertKAPRecMemory(Vert_Ptr->Element);
	}
}


// PRE	TRUE
// POST	All memory allocated to nodes in VertBST is freed
void
VertBST_Dispose_All_Nodes_R(VertBST_T &VertBST) {
	VertNode_Ptr_T	Vert_Ptr;

	Vert_Ptr = (VertNode_Ptr_T) VertBST;

	if (Vert_Ptr != NULL) {
		VertBST_Dispose_All_Nodes_R((VertBST_T) Vert_Ptr->Left_Ptr);
		VertBST_Dispose_All_Nodes_R((VertBST_T) Vert_Ptr->Right_Ptr);
		delete(Vert_Ptr);
	}
}


// PRE	TRUE
// POST	All memory for nodes allocated to VertBST is freed
//			AND Q_EmptyVertTree(VertBST')
// NOTES	This will not dispose of memory referenced by the elements
//			e.g. through KAPRec_T's
void
VertBST_Delete_All(VertBST_T &VertBST) {

	VertBST_Dispose_All_Nodes_R(VertBST);
	VertBST = NULL;
}


// PRE	TRUE
// POST	Iterator is created
VertBST_Iterator_T
Create_VertIterator() {
	VertBST_ItRec_Ptr_T ItRec;

	ItRec = new(VertBST_ItRec_T);
	ItRec->Stack = Create_Stack();
	ItRec->Q_Bound = FALSE;
	return(ItRec);
}

// PRE	Create_Iterator(Iterator) has been called
//			AND since last, if any, call to
//			Destroy_VertIterator(Iterator)
// POST	All memory for Iterator is freed up
void
Destroy_VertIterator(VertBST_Iterator_T Iterator) {
	VertBST_ItRec_Ptr_T ItRec;

	ItRec = (VertBST_ItRec_Ptr_T) Iterator;
	for(;!Q_EmptyStack(ItRec->Stack);) {
		Pop(ItRec->Stack);
	}
	Free_Stack(ItRec->Stack);
}

// PRE	TRUE
// POST	IF Start_Iteration(Iterator, VertBST) has been executed
//				correctly for some value VertBST
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
Q_Bound_To_VertBST(VertBST_Iterator_T Iterator) {
	VertBST_ItRec_Ptr_T ItRec;

	ItRec = (VertBST_ItRec_Ptr_T) Iterator;
	return(ItRec->Q_Bound);
}


// PRE	TRUE
// POST	IF iteration is at ordinally keyed first vert
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_StartOfVertIteration(VertBST_Iterator_T Iterator) {
	VertBST_ItRec_Ptr_T ItRec;

	ItRec = (VertBST_ItRec_Ptr_T) Iterator;
	return(ItRec->CurrentNumber = 1);
}


// PRE	Q_Bound_To_VertBST(Iterator)
// POST	IF iteration is at ordinally keyed last vert
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_EndOfVertIteration(VertBST_Iterator_T Iterator) {
	VertBST_ItRec_Ptr_T ItRec;
	VertNode_Ptr_T CurrentVert;

	ItRec = (VertBST_ItRec_Ptr_T) Iterator;
	CurrentVert = (VertNode_Ptr_T) ItRec->Current;
	return(	(CurrentVert->Right_Ptr == NULL) &&
				Q_EmptyStack(ItRec->Stack));
}


// PRE	NOT Q_EmptyTree(VertBST)
// POST	Iterator is reset with
//			ordinally first key vert as current vert
void
Start_VertIteration(VertBST_Iterator_T &Iterator, VertBST_T VertBST) {
	VertBST_ItRec_Ptr_T ItRec;
	VertNode_Ptr_T Vert_Ptr;

	ItRec = (VertBST_ItRec_Ptr_T) Iterator;
	for (;!Q_EmptyStack(ItRec->Stack);) {
		 Pop(ItRec->Stack);
	}
	Vert_Ptr = (VertNode_Ptr_T) VertBST;
	for (;Vert_Ptr->Left_Ptr != NULL;) {
		Push(Vert_Ptr, ItRec->Stack);
		Vert_Ptr = Vert_Ptr->Left_Ptr;
	}
	// Vert_Ptr is the leftmost vert of VertBST
	ItRec->Current = (VertBST_T) Vert_Ptr;
	ItRec->CurrentNumber = 1;
	ItRec->Q_Bound = TRUE;
}


// PRE	Start_Iteration(&Iterator, VertBST) has been called
//			for some value VertBST
//			AND NOT Q_EndOfIteration(Iterator)
// POST	NOT Q_Iteration_Start(Iterator') AND
//					current is next node in iteration
void
VertIterate(VertBST_Iterator_T &Iterator) {
	VertBST_ItRec_Ptr_T ItRec;
	VertNode_Ptr_T Current;

	ItRec = (VertBST_ItRec_Ptr_T) Iterator;
	(ItRec->CurrentNumber)++;
	Current = (VertNode_Ptr_T) ItRec->Current;
	if (Current->Right_Ptr == NULL) {
		 ItRec->Current = Top(ItRec->Stack);
		 Pop(ItRec->Stack);
	}
	else {
		 Current = Current->Right_Ptr;
		 for (;(Current->Left_Ptr != NULL);) {
			  Push((VertBST_T) Current, ItRec->Stack);
			  Current = Current->Left_Ptr;
		 }
		ItRec->Current = (VertBST_T) Current;
	}
}


// PRE     Q_Iteration_Started(Iterator)
// RETURNS current record in iteration
VertKAPRec_T
Get_CurrentVert(VertBST_Iterator_T Iterator) {
	VertBST_ItRec_Ptr_T ItRec;
	VertNode_Ptr_T CurrentVert;

	ItRec = (VertBST_ItRec_Ptr_T) Iterator;
	CurrentVert = (VertNode_Ptr_T) ItRec->Current;
	return(CurrentVert->Element);
}


// PRE	Q_Iteration_Started(Iterator)
//			AND Get_VertKAPRecKey(VertKAPRecUpdate) = Get_VertKAPRecKey(Get_CurrentVert(Iterator))
// POST	Get_CurrentVert(Iterator) = VertKAPRecUpdate))
//	NOTES	PRE (ii) required to maintain key ordering
void
Update_CurrentVert(VertKAPRec_T VertKAPRecUpdate, VertBST_Iterator_T Iterator) {
	VertBST_ItRec_Ptr_T ItRec;

	ItRec = (VertBST_ItRec_Ptr_T) Iterator;
	((VertNode_Ptr_T) ItRec->Current)->Element = VertKAPRecUpdate;
}


// PRE	Q_Iteration_Started(Iterator)
// POST	RETURNS number of current record in iteration
int
Get_CurrentVertNumber(VertBST_Iterator_T Iterator) {
	VertBST_ItRec_Ptr_T ItRec;

	ItRec = (VertBST_ItRec_Ptr_T) Iterator;
	return(ItRec->CurrentNumber);
}


