// two_way.mod
// 1/8/98
// TwoWay_T implementation based on doubly linked list
// Ken Brownsey
#include "two_way.h"


typedef struct Node_Tag {
	struct	Node_Tag	*Prev_Ptr,
							*Next_Ptr;
	Element_T	Element;
} Node_T, *Node_Ptr_T;

typedef struct {
	Node_Ptr_T	Head,
					Current;
}	DLLRec_T, *DLLRec_Ptr_T;


// PRE	TRUE
// POST	RETURNS an initialised TwoWay_T
TwoWay_T
Create_TwoWay() {
	DLLRec_Ptr_T DLL;

	DLL = new DLLRec_T;
	DLL->Head = NULL;
	DLL->Current = NULL;
	return((TwoWay_T) DLL);
}

// PRE	TRUE
//	POST	if TWSeq is full {
//				RETURNS TRUE
//			else { RETURNS FALSE
BOOLEAN
Q_Full(TwoWay_T TWSeq) {
	return(FALSE);
}


// PRE	TRUE
//	POST	if TWSeq is empty { RETURNS TRUE
//			else { RETURNS FALSE
BOOLEAN
Q_Empty(TwoWay_T TWSeq) {
	DLLRec_Ptr_T DLL;

	DLL = (DLLRec_Ptr_T) TWSeq;
	return(DLL->Head == NULL);
}



// PRE	NOT Q_Empty(TWSeq)
//	POST	if current element first in TWSeq {
//				RETURNS TRUE
//			else { RETURNS FALSE
BOOLEAN
Q_At_Start(TwoWay_T TWSeq) {
	DLLRec_Ptr_T DLL;

	DLL = (DLLRec_Ptr_T) TWSeq;
	return(DLL->Current->Prev_Ptr == NULL);
}


// PRE	NOT Q_Empty(TWSeq)
//	POST	if current element last in TWSeq {
//				RETURNS TRUE
//			else { RETURNS FALSE
BOOLEAN
Q_At_End(TwoWay_T TWSeq) {
	DLLRec_Ptr_T DLL;

	DLL = (DLLRec_Ptr_T) TWSeq;
	return(DLL->Current->Next_Ptr == NULL);
}


// PRE	NOT Q_Empty(TWSeq)
//	POST	first element of TWSeq' is the current element
void
Go_To_Head(TwoWay_T TWSeq) {
	DLLRec_Ptr_T DLL;

	DLL = (DLLRec_Ptr_T) TWSeq;
	DLL->Current = DLL->Head;
}


// PRE	NOT Q_Empty(TWSeq) AND
//			NOT Q_At_End(TWSeq)
//	POST	current element in TWSeq' is the one after
//			current element in TWSeq
void
Go_To_Next(TwoWay_T TWSeq) {
	DLLRec_Ptr_T DLL;

	DLL = (DLLRec_Ptr_T) TWSeq;
	DLL->Current = DLL->Current->Next_Ptr;
}


// PRE	NOT Q_Empty(TWSeq) AND
//			NOT Q_At_Start(TWSeq)
//	POST	current element in TWSeq' is the one before
//			current element in TWSeq
void
Go_To_Prev(TwoWay_T TWSeq) {
	DLLRec_Ptr_T DLL;

	DLL = (DLLRec_Ptr_T) TWSeq;
	DLL->Current = DLL->Current->Prev_Ptr;
}

// PRE	NOT Q_Empty(TWSeq)
// POST	RETURNS current record
Element_T
Get_Current(TwoWay_T TWSeq) {
	DLLRec_Ptr_T DLL;

	DLL = (DLLRec_Ptr_T) TWSeq;
	return(DLL->Current->Element);
}



// PRE	TRUE
// POST	TWSeq'== TWSeq with NewRec added AND
//			if Q_Empty(TWSeq) {
//				current of TWSeq' is first element
//			else {
//				NewRec is after current in TWSeq'
void
Append_After_Current(Element_T NewRec, TwoWay_T TWSeq) {
	Node_Ptr_T NewNode_Ptr;
	DLLRec_Ptr_T DLL;

	DLL = (DLLRec_Ptr_T) TWSeq;
	NewNode_Ptr = new Node_T;
	NewNode_Ptr->Element = NewRec;

	if (Q_Empty(DLL)) {
		DLL->Current = NewNode_Ptr;
		DLL->Head = NewNode_Ptr;
		DLL->Current->Prev_Ptr = NULL;
		DLL->Current->Next_Ptr = NULL;
	}
	else {
		NewNode_Ptr->Next_Ptr = DLL->Current->Next_Ptr;
		NewNode_Ptr->Prev_Ptr = DLL->Current;
		DLL->Current->Next_Ptr = NewNode_Ptr;
		if (NewNode_Ptr->Next_Ptr != NULL) {
			NewNode_Ptr->Next_Ptr->Prev_Ptr = NewNode_Ptr;
	  }
	}
}


// PRE	NOT Q_Empty(DLL)
// POST	TWSeq'= TWSeq with NewRec added AND
//			NewRec is before current in TWSeq'
void
Insert_Before_Current(Element_T NewRec, TwoWay_T TWSeq) {
	Node_Ptr_T NewNode_Ptr;
	DLLRec_Ptr_T DLL;

	DLL = (DLLRec_Ptr_T) TWSeq;
	NewNode_Ptr = new Node_T;
	NewNode_Ptr->Element = NewRec;

	NewNode_Ptr->Next_Ptr = DLL->Current;
	NewNode_Ptr->Prev_Ptr = DLL->Current->Prev_Ptr;
	if (DLL->Current->Prev_Ptr != NULL) {
		DLL->Current->Prev_Ptr->Next_Ptr = NewNode_Ptr;
		DLL->Current->Prev_Ptr = NewNode_Ptr;
	}
	else {
	// Current is Head
		DLL->Current->Prev_Ptr = NewNode_Ptr;
		DLL->Head = NewNode_Ptr;
	}
}


// PRE	NOT Q_Empty(TWSeq)
// POST	TWSeq'= TWSeq with current of TWSeq removed AND
//			if NOT(Q_Empty(TWSeq'))
//				current of TWSeq' is first element of TWSeq'
void
Delete_Current(TwoWay_T TWSeq) {
	DLLRec_Ptr_T DLL;

	DLL = (DLLRec_Ptr_T) TWSeq;
	if (DLL->Head == DLL->Current) {
		DLL->Current = DLL->Current->Next_Ptr;
		if (DLL->Current != NULL) {
			DLL->Current->Prev_Ptr = NULL;
		}
		delete(DLL->Head);
		DLL->Head = DLL->Current;
	}
	else {
		DLL->Current->Prev_Ptr->Next_Ptr = DLL->Current->Next_Ptr;
		if (DLL->Current->Next_Ptr != NULL) {
			DLL->Current->Next_Ptr->Prev_Ptr = DLL->Current->Prev_Ptr;
		}
		delete(DLL->Current);
		DLL->Current = DLL->Head;
	}
}


//	PRE	Q_Empty(TWSeq)
//	POST	all memory allocated to TWSeq is freed up
// NOTES	TWSeq will need to be re-initialised
//			with Create_TWSeq() before being used again
void
Free_TWSeq(TwoWay_T TWSeq) {
	DLLRec_Ptr_T DLL;

	DLL = (DLLRec_Ptr_T) TWSeq;
	delete(DLL);
}

