// Ken Brownsey
// 17/1/96
// lex.c
// lexing 'object'


#include "lex.h"
#include <iostream.h>
#include <ctype.h>


struct {
	BOOLEAN	Q_OK_Lex;
	TokenType_T	TokenType;
	char	CharVal;
	int	IntVal;
}	TokenRecord;

char	CurrChar;


// PRE TRUE
// POST	CurrChar the next character from stdin
void
Start_Lex() {
		cin.get(CurrChar);
}


// PRE TRUE
// POST	CurrChar is the next newline
void
GoTo_NewLine() {

	for(; CurrChar != NL_C;) {
		cin.get(CurrChar);
	}
}


// PRE	GoTo_NewLine executed immediately before
// POST	CurrChar contains the first character on the
// 	next proper - i.e. non-empty - new line
//		and this was the last chracter from standard input
void
Start_NewLine() {

	for(; CurrChar == NL_C;) {
		cin.get(CurrChar);
	}
}


// PRE TRUE
// POST 	recursively reads in digits
//			AND updates the value of TokenRecord.IntVal
void
Int_Lex() {
	if (isdigit(CurrChar)) {
		TokenRecord.IntVal =
			TokenRecord.IntVal * 10 + ((int) CurrChar - ((int) '0'));
		cin.get(CurrChar);
		Int_Lex();
	}
}


// PRE	-	TRUE
// POST	-	IF there is a valid token on input THEN
//					token is read in
//					AND Get_TokenType returns its type
//					AND Q_LexOK() will return TRUE
//				ELSE Q_LexOK() will return FALSE
void
Lex_Token() {
	TokenRecord.Q_OK_Lex = TRUE;

	if (CurrChar == LRB_C) {
		TokenRecord.TokenType = LRB_TT;
		cin.get(CurrChar);
	}

	else if (CurrChar == RRB_C) {
		TokenRecord.TokenType = RRB_TT;
		cin.get(CurrChar);
	}

	else if (CurrChar == SemiCol_C) {
		TokenRecord.TokenType = SemiCol_TT;
		cin.get(CurrChar);
	}

	else if ((CurrChar == Plus_C) ||
				(CurrChar == Minus_C) ||
				(CurrChar == Mult_C) ||
				(CurrChar == Div_C)) {
		TokenRecord.TokenType = Operator_TT;
		TokenRecord.CharVal = CurrChar;
		cin.get(CurrChar);
	}

	else if (isdigit(CurrChar)) {
		TokenRecord.IntVal = 0;
		Int_Lex();
		TokenRecord.TokenType = INT_TT;
	}

	else if ((CurrChar == Space_C) ||
				(CurrChar == Tab_C)) {
		TokenRecord.TokenType = Separator_TT;
		TokenRecord.CharVal = CurrChar;
		cin.get(CurrChar);
	}

	else if (CurrChar == NewLine_C) {
		TokenRecord.TokenType = EndOfLine_TT;
		cin.get(CurrChar);
	}

	else {
		TokenRecord.Q_OK_Lex = FALSE;
		TokenRecord.TokenType = NULL_TT;
		TokenRecord.CharVal = NULL_C;
		cin.get(CurrChar);
	}
}


// PRE	-	Q_LexOK()
// POST	-	RETURNS type of last token
TokenType_T
Get_TokenType() {
	return(TokenRecord.TokenType);
}


// PRE	-	Q_LexOK() AND type of last token is character
//			e.g Operator_TT
// POST	-	RETURNS value of last token
char
Get_CharVal() {
	return(TokenRecord.CharVal);
}


// PRE	-	Q_LexOK() AND type of last token is integer
// POST	-	RETURNS value of last token
int
Get_IntVal() {
	return(TokenRecord.IntVal);
}


// PRE	-	Lex_Token() has been called
// POST	-	IF a valid token was read THEN
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_LexOK() {
	return(TokenRecord.Q_OK_Lex);
}