// hanoi.cpp
// 16/8/98
// Demonstrates recursion in Hanoi problem solving
//	with Element_T = int
// Ken Brownsey

#include <iostream.h>
#include "stack.h"

typedef Stack_T Hanoi_T[3];
typedef enum Peg_T {A, B, C};
typedef enum Direction_T {ClockWise, AntiClockWise};
typedef Element_T Disc_T;

// SECTION 1 - Operations on Peg_T ADT

// PRE	TRUE
// POST	RETURNS next peg after peg in direction Direction
Peg_T
Next_Peg(Peg_T Peg, Direction_T Direction) {
	if (Direction == ClockWise) {
		return((Peg + 1)%3);
	}
	else {
		return((Peg + 2)%3);
	}
}


// PRE	TRUE
// POST	RETURNS character for peg is written to screen
char
Char_For_Peg(Peg_T Peg) {
	return((char)(int('A') + Peg));
}


// PRE 	TRUE
// POST  Moving Disc from FromPeg to ToPeg shown on screen
//			0 ==> A, 1 ==> B, 2 ==> C for pegs
void
Show_Move(Disc_T NextDisc, Peg_T FromPeg, Peg_T ToPeg) {

	cout	<< "\nMoving disc ";
			Write_Rec(NextDisc);
	cout	<< " from peg "
			<< Char_For_Peg(FromPeg)
			<< " to peg "
			<< Char_For_Peg(ToPeg);
}



// ENDSECTION 1 - Operations on Peg_T ADT



// SECTION 2 - Operations on Stack_T ADT

// PRE	TRUE
// POST	Stack printed out in order of recentness
/*void
Print_Stack_R(Stack_T Stack) {
	Element_T NextEl;

	if (!Q_EmptyStack(Stack)) {
		NextEl = Top(Stack);
		Write_Rec(NextEl);
		cout	<< " ";
		Pop(Stack);
		Print_Stack_R(Stack);
		Push(NextEl, Stack);
	}
} */
// PRE	TRUE
// POST	Stack printed out in order of recentness
void
Print_Stack(Stack_T Stack) {
	Stack_T	Temp;
	char	NextChar;

	Temp = Create_Stack();
	for(;!Q_EmptyStack(Stack);) {
		NextChar = Top(Stack);
		Push(NextChar, Temp);
		Pop(Stack);
		Write_Rec(NextChar);
		cout	<< ' ';
	}
	for(;!Q_EmptyStack(Temp);) {
		NextChar = Top(Temp);
		Push(NextChar, Stack);
		Pop(Temp);
	}
	Free_Stack(Temp);
}


// END SECTION 2 - Operations on Stack_T ADT


// SECTION 3 - Operations on Hanoi_T ADT



// PRE	Peg1 and Peg2 of Hanoi not empty
// POST	IF disc on Peg1 bigger than that on Peg2
//				RETURNS TRUE
// 		ELSE
//				RETURNS FALSE
BOOLEAN
Q_BiggerDisc(Peg_T Peg1, Peg_T Peg2, Hanoi_T Hanoi) {

	return(Top(Hanoi[Peg1]) > Top(Hanoi[Peg2]));
}


// PRE	Peg1 and Peg2 of Hanoi not empty
// POST	IF disc on Peg1 bigger than that on Peg2
//				RETURNS TRUE
// 		ELSE
//				RETURNS FALSE
BOOLEAN
Q_EmptyPeg(Peg_T Peg, Hanoi_T Hanoi) {

	return(Q_EmptyStack(Hanoi[Peg]));
}


// PRE	TRUE
// POST	Towers of Hanoi printed to screen (left is top)
void
Print_Hanoi(Hanoi_T Hanoi) {
	Peg_T	Index;

	for(Index = A; Index <= C; Index++) {
		cout	<< "Peg "
				<< Char_For_Peg(Index)
				<< " ==> ";
		Print_Stack(Hanoi[Index]);
		cout	<< "\n";
	}
}



// PRE 	TRUE
// POST  Top disc moved from FromPeg to ToPeg
//			AND message written to screen
//			0 ==> A, 1 ==> B, 2 ==> C for pegs
void
Move_Disc(Peg_T FromPeg, Peg_T ToPeg, Hanoi_T Hanoi) {

	Disc_T	Disc;

	Disc = Top(Hanoi[FromPeg]);
	Pop(Hanoi[FromPeg]);
	Push(Disc, Hanoi[ToPeg]);
	Show_Move(Disc, FromPeg, ToPeg);
}


// PRE	TRUE
// POST	Hanoi with NumDiscs set up as problem
void
Reset_Hanoi(Hanoi_T Hanoi, int NumDiscs) {
	Disc_T Index;

	for (; !Q_EmptyStack(Hanoi[A]); ) {
		Pop(Hanoi[A]);
	}
	for (; !Q_EmptyStack(Hanoi[B]); ) {
		Pop(Hanoi[B]);
	}
	for (; !Q_EmptyStack(Hanoi[C]); ) {
		Pop(Hanoi[C]);
	}

	for (Index = NumDiscs; Index > 0; Index--) {
		Push(Index, Hanoi[0]);
	}
}


// END SECTION 3 - Operations on Hanoi_T ADT


