// han_demo.cpp
// 16/8/98
// Demonstrates Hanoi recursive and iterative problem solving
//	with Element_T = int
// Ken Brownsey

#include "hanoi.h"

// PRE	TRUE
// POST  Fills up first stack for Hanoi
void
Set_Up_Hanoi(Hanoi_T Hanoi, int &NumDiscs, BOOLEAN &StopAndDisplay) {
	char	Response;

	cout	<< "\nHow many discs ==> ";
	cin	>> NumDiscs;

	cout	<< "\nDo you want to stop and display every four moves ? (y/n) ==> ";
	cin	>> Response;
	if (Response == 'y') {
		StopAndDisplay = TRUE;
	}
	else {
		StopAndDisplay = FALSE;
	}

	Reset_Hanoi(Hanoi, NumDiscs);
	Print_Hanoi(Hanoi);
	cout	<< "\n";
}



// PRE	Reset(Hanoi, NumDiscs) has been called since the last call to
//			Do_Hanoi_R(Hanoi...) or It_Hanoi(Hanoi...)
//	POST	Solution to Hanoi problem shown on screen with moves and peg states
void
Do_Hanoi_R(	int NumDiscs, Peg_T From, Peg_T Via,
				Peg_T To, Hanoi_T Hanoi,
				BOOLEAN StopAndDisplay, int &Moves) {
	char	Dummy;

	if (NumDiscs > 0) {
		Do_Hanoi_R(NumDiscs - 1, From, To, Via, Hanoi, StopAndDisplay, Moves);
		Move_Disc(From, To, Hanoi);
		cout	<< "\n";
		Print_Hanoi(Hanoi);
		if (StopAndDisplay) {
			Moves++;
			if (Moves == 4) {
				cout	<<	"\nPress ENTER to continue\n";
				cin.get(Dummy);
				Moves = 0;
			}
		}
		Do_Hanoi_R(NumDiscs - 1, Via, From, To, Hanoi, StopAndDisplay, Moves);
	}
}


// PRE	Num > 1 AND Exp >=0
// POST	RETURNS Num raised to Exp
//			e.g. Power(5, 3) RETURNS 125
int
Power(int Num, int Exp) {
	if (Exp == 0) {
		return(1);
	}
	else {
		return(Num * Power(Num, Exp - 1));
	}
}


// PRE	Reset(Hanoi, NumDiscs) has been called since the
//			last call to Do_Hanoi_R(Hanoi...) or It_Hanoi(Hanoi...)
//	POST	Solution to Hanoi problem shown on screen with moves
void
It_Hanoi(Hanoi_T Hanoi, int NumDiscs, BOOLEAN StopAndDisplay, int Moves) {
	int 	Index,
			NumMoves;
	Direction_T	Direction;
	Peg_T	CurrPeg,
			Peg1,
			Peg2,
			From,
			To;
	char	Dummy;

	if (NumDiscs%2 == 0) {
		Direction = ClockWise;
	}
	else {
		Direction = AntiClockWise;
	}
	NumMoves = Power(2, NumDiscs) - 1;
	cout	<< "\nThere will be "
			<< NumMoves
			<< " moves";

	CurrPeg = A;
	for(Index = 1; Index <= NumMoves; Index++) {
		if (Index%2 == 1) {
			From = CurrPeg;
			To = Next_Peg(From, Direction);
			CurrPeg = To;
			Move_Disc(From, To, Hanoi);
		}
		else {
			Peg1 = Next_Peg(CurrPeg, Direction);
			Peg2 = Next_Peg(Peg1, Direction);
			if (Q_EmptyPeg(Peg1, Hanoi)) {
				From = Peg2;
				To = Peg1;
			}
			else {
				if (Q_EmptyPeg(Peg2, Hanoi)) {
					From = Peg1;
					To = Peg2;
				}
				else {
					if (Q_BiggerDisc(Peg1, Peg2, Hanoi)) {
						From = Peg2;
						To = Peg1;
					}
					else {
						From = Peg1;
						To = Peg2;
					}
				}
			}
			Move_Disc(From, To, Hanoi);
		}
		cout	<< "\n";
		Print_Hanoi(Hanoi);
		if (StopAndDisplay) {
			Moves++;
			if (Moves == 4) {
				cout	<<	"\nPress ENTER to continue\n";
				cin.get(Dummy);
				Moves = 0;
			}
		}
	}
}


// PRE  -	TRUE
// POST -	menu displayed
void
Menu() {
	cout	<<	"***	(s)et up Hanoi       ****************\n"
			<< "***	do (h)anoi showing towers ***********\n"
			<< "***	do (i)terative Hanoi showing towers *\n"
			<< "***  (m)enu						  ***********\n";
}


// PRE	TRUE
// POST	Initialisation is done
void
Initialise(Hanoi_T &Hanoi) {

	Hanoi[0] = Create_Sequence();
	Hanoi[1] = Create_Sequence();
	Hanoi[2] = Create_Sequence();
	cout	<< "Here we go !\n";
}


// PRE	TRUE
// POST Goes into a prompt, read and execute cycle
//		until 'x' is the character read
void
Loop(Hanoi_T Hanoi) {

	char	Request;
	int	RequestCount;
	int	NumDiscs;
	BOOLEAN StopAndDisplay;
	int	Moves;
	char	Dummy;

	Set_Up_Hanoi(Hanoi, NumDiscs, StopAndDisplay);
	for (Request = 'm', RequestCount = 0; Request != 'x'; RequestCount++) {

		switch(Request) {

			case 's' :	Set_Up_Hanoi(Hanoi, NumDiscs, StopAndDisplay);
				break;
			case 'h' :	Moves = 0;
							cout	<<	"\nPress ENTER to continue\n";
							cin.get(Dummy);
							Do_Hanoi_R(NumDiscs, A, B, C, Hanoi, StopAndDisplay, Moves);
							Reset_Hanoi(Hanoi, NumDiscs);
				break;
			case 'i' :	Moves = 0;
							cout	<<	"\nPress ENTER to continue\n";
							cin.get(Dummy);
							It_Hanoi(Hanoi, NumDiscs, StopAndDisplay, Moves);
							Reset_Hanoi(Hanoi, NumDiscs);
				break;
			case 'c' :	StackStats();
				break;
			case 'm' : Menu();
				break;
			default   :
				cout	<< "\n m for menu \n";
		}
		cout	<< "\n\nNext request ["
				<< RequestCount
				<< "] ==> ";
		cin	>> Request;
	}
}


// PRE	TRUE
// POST	Clears up
void
Finalise(Hanoi_T Hanoi) {
	for(;!Q_Empty(Hanoi[C]);) {
		Delete_Current(Hanoi[C]);
	}
	Free_Stack(Hanoi[A]);
	Free_Stack(Hanoi[B]);
	Free_Stack(Hanoi[C]);
	cout	<< "That is it !\n";
}


void
main() {
	Hanoi_T	Hanoi;

	Initialise(Hanoi);
	Loop(Hanoi);
	Finalise(Hanoi);
}