// sequence.cpp
// 1/8/98
// Sequence_T implementation based on linked list
// Ken Brownsey

#include "sequence.h"
#include <stddef.h>

typedef struct Node_Tag {
	struct Node_Tag		*Next_Ptr;
	Element_T		Element;
} Node_T, *Node_Ptr_T;


typedef struct {
	Node_Ptr_T	Head,
					Current,
					BeforeCurrent;
}	SeqRec_T, *SeqRec_Ptr_T;


// PRE	TRUE
// POST	RETURNS an initialised Sequence_T
Sequence_T
Create_Sequence() {
	SeqRec_Ptr_T NewSeq;

	NewSeq = new SeqRec_T;
	NewSeq->Head = NULL;
	NewSeq->Current = NULL;
	NewSeq->BeforeCurrent = NULL;
	return((Sequence_T) NewSeq);
}


// PRE	TRUE
//	POST	IF Sequence is full THEN
//				RETURNS TRUE
//			ELSE RETURNS FALSE
BOOLEAN
Q_Full(Sequence_T Sequence) {
	return(FALSE);
}


// PRE	TRUE
//	POST	IF Sequence is empty THEN RETURNS TRUE
//			ELSE RETURNS FALSE
BOOLEAN
Q_Empty(Sequence_T Sequence) {
	SeqRec_Ptr_T Seq;

	Seq = (SeqRec_Ptr_T) Sequence;
	return(Seq->Head == NULL);
}


// PRE	NOT Q_Empty(Sequence)
//	POST	IF current element last in Sequence THEN
//				RETURNS TRUE
//			ELSE RETURNS FALSE
BOOLEAN
Q_At_End(Sequence_T Sequence){
	SeqRec_Ptr_T Seq;

	Seq = (SeqRec_Ptr_T) Sequence;
	return(Seq->Current->Next_Ptr == NULL);
}


// PRE	NOT Q_Empty(Sequence)
//	POST	first element of Sequence' is the current element
void
Go_To_Head(Sequence_T Sequence) {
	SeqRec_Ptr_T Seq;

	Seq = (SeqRec_Ptr_T) Sequence;
	Seq->Current = Seq->Head;
	Seq->BeforeCurrent = NULL;
}


// PRE	NOT Q_Empty(Sequence) AND
//			NOT Q_At_End(Sequence)
//	POST	current element in Sequence' is the one after
//			current element in Sequence
void
Go_To_Next(Sequence_T Sequence) {
	SeqRec_Ptr_T Seq;

	Seq = (SeqRec_Ptr_T) Sequence;
	Seq->BeforeCurrent =	Seq->Current;
	Seq->Current =	Seq->Current->Next_Ptr;
}


// PRE	NOT Q_Empty(Sequence)
// POST	RETURNS current record
Element_T
Get_Current(Sequence_T Sequence) {
	SeqRec_Ptr_T Seq;

	Seq = (SeqRec_Ptr_T) Sequence;
	return(Seq->Current->Element);
}


// PRE	TRUE
// POST	Sequence' = Sequence with NewRec added AND
//			IF Q_Empty(Sequence) THEN
//				current of Sequence' is first element
//			ELSE
//				NewRec is after current in Sequence'
void
Append_After_Current(Element_T NewRec, Sequence_T Sequence) {
	SeqRec_Ptr_T 	Seq;
	Node_Ptr_T     NewNode_Ptr;

	Seq = (SeqRec_Ptr_T) Sequence;

	NewNode_Ptr = new Node_T;
	NewNode_Ptr->Element = NewRec;

	if (Q_Empty(Seq)) {
		Seq->Current = NewNode_Ptr;
		Seq->Head = NewNode_Ptr;
		Seq->Current->Next_Ptr = NULL;
	}
	else {
		NewNode_Ptr->Next_Ptr = Seq->Current->Next_Ptr;
		Seq->Current->Next_Ptr = NewNode_Ptr;
	}
}


// PRE	NOT Q_Empty(Sequence)
// POST	Sequence' = Sequence with NewRec added AND
//			NewRec is before current in Sequence'
void
Insert_Before_Current(Element_T NewRec, Sequence_T Sequence) {
	SeqRec_Ptr_T Seq;
	Node_Ptr_T     NewNode_Ptr;

	Seq = (SeqRec_Ptr_T) Sequence;

	NewNode_Ptr = new Node_T;
	NewNode_Ptr->Element = NewRec;

	NewNode_Ptr->Next_Ptr = Seq->Current;
	if (Seq->BeforeCurrent != NULL) {
		Seq->BeforeCurrent->Next_Ptr = NewNode_Ptr;
		Seq->BeforeCurrent = NewNode_Ptr;
	}
	else {
	// Current is Head
		Seq->BeforeCurrent = NewNode_Ptr;
		Seq->Head = NewNode_Ptr;
	}
}


// PRE	NOT Q_Empty(Sequence)
// POST	Sequence' = Sequence with current of Sequence removed AND
//			IF NOT(Q_Empty(Sequence'))
//				current of Sequence' is first element of Sequence'
void
Delete_Current(Sequence_T Sequence) {
	SeqRec_Ptr_T Seq;

	Seq = (SeqRec_Ptr_T) Sequence;

	if (Seq->Head == Seq->Current) {
		Seq->Current = Seq->Current->Next_Ptr;
      delete Seq->Head;
		Seq->Head = Seq->Current;
	}
	else {
		Seq->BeforeCurrent->Next_Ptr = Seq->Current->Next_Ptr;
		Seq->BeforeCurrent = NULL;
		delete Seq->Current;
		Seq->Current = Seq->Head;
	}
}


//	PRE	Q_Empty(Sequence)
//	POST	all memory allocated to Sequence is freed up
// NOTES	Sequence will need to be re-initialised
//			with Create_Sequence() before being used again
void
Free_Sequence(Sequence_T Sequence) {
	SeqRec_Ptr_T Seq;

	Seq = (SeqRec_Ptr_T) Sequence;
	delete(Seq);
}


void
Junk() {
	Node_Ptr_T     NewNode_Ptr;

	NewNode_Ptr = new Node_T;

}

