// Ken Brownsey
// 31/8/98
// bst_tree.cpp
// BST_T directly pointer based implementation

#include "my_const.h"
#include "bst_tree.h"

// node structure for linked tree
typedef struct TN_T {
	struct TN_T	*Left_Ptr,
					*Right_Ptr;
	Element_T	Element;
} Node_T, *Node_Ptr_T;

typedef Node_Ptr_T *Node_Ptr_Ptr_T;

int	NumberAllocated = 0;


// PRE	TRUE
// POST	RETURNS the empty tree
BST_T
Create_Tree() {
	return(NULL);
}


// PRE	TRUE
// POST	IF Tree is empty RETURNS TRUE ELSE RETURNS FALSE
BOOLEAN
Q_EmptyTree(BST_T Tree) {
	return(Tree == NULL);
}


// PRE  	Tree is not empty
// POST	RETURNS the root record
Element_T
Get_Root(BST_T Tree) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) Tree;

	return(Node_Ptr->Element);
}


// PRE	TRUE
// POST	IF Tree is not empty
//				RETURNS the left sub tree of Tree
//			ELSE
//				RETURNS the empty tree
BST_T
Get_Left(BST_T Tree) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) Tree;

	if (Node_Ptr == NULL) {
		return(NULL);
	}
	else {
		return((void *) Node_Ptr->Left_Ptr);
	}
}


// PRE	TRUE
// POST	IF Tree is not empty
//				RETURNS the right sub tree of Tree
//			ELSE
//				RETURNS the empty tree
BST_T
Get_Right(BST_T Tree) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) Tree;

	if (Node_Ptr == NULL) {
		return(NULL);
	}
	else {
		return((void *) Node_Ptr->Right_Ptr);
	}
}


// PRE	TRUE
// POST	RETURNS the tree whose root is Root,
//				left sub tree is Left,
//				right sub tree is Right
BST_T
Make_Tree(BST_T Left, Element_T Element, BST_T Right) {

	Node_Ptr_T	NewTree;

	NewTree = new Node_T;
	NumberAllocated++;
	NewTree->Element = Element;
	NewTree->Left_Ptr = (Node_Ptr_T) Left;
	NewTree->Right_Ptr = (Node_Ptr_T) Right;
	return((void *) NewTree);
}


// PRE  - 	TRUE
// POST - 	RETURNS the number of nodes allocated
Get_NumberNodes_Allocated() {
	return(NumberAllocated);
}


// PRE	TRUE
// POST	if record with key Key is in BST
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_BST_Present_R(Key_T Key, BST_T BST) {
	if (Q_EmptyTree(BST)) {
		return(FALSE);
	}
	else {
		if (Key < Get_Key(Get_Root(BST))) {
			return(Q_BST_Present_R(Key, Get_Left(BST)));
		}
		else {
			if (Key > Get_Key(Get_Root(BST))) {
				return(Q_BST_Present_R(Key, Get_Right(BST)));
			}
			else {
				return(TRUE);
			}
		}
	}
}


// PRE	NOT Q_BST_Present(Rec, BST)
// POST	BST' is BST with Rec inserted
void
BST_Insert(Element_T Rec, BST_T &BST) {
	Key_T Key;
	Node_Ptr_T 	NewNode_Ptr,
					Parent_Ptr;
	BOOLEAN	Q_OKInsert;

	NewNode_Ptr = new(Node_T);
	NewNode_Ptr->Left_Ptr 	= NULL;
	NewNode_Ptr->Element 	= Rec;
	NewNode_Ptr->Right_Ptr 	= NULL;
	if (BST == NULL) {
		BST = NewNode_Ptr;
	}
	else {
		Parent_Ptr = (Node_Ptr_T) BST;
		Q_OKInsert = FALSE;
		Key = Get_Key(Rec);
		for(;!Q_OKInsert;) {
			if (Key < Get_Key(Parent_Ptr->Element)) {
			// look ahead left
				if (Parent_Ptr->Left_Ptr == NULL) {
				// insertion must be in node referenced by Parent_Ptr
					Parent_Ptr->Left_Ptr = NewNode_Ptr;
					Q_OKInsert = TRUE;
				}
				else {
				// node referenced by Parent_Ptr is not the last node
				// on the way to where NewNode_Ptr is to be inserted
					Parent_Ptr = Parent_Ptr->Left_Ptr;
				}
			}
			else {
			// look ahead right
				if (Parent_Ptr->Right_Ptr == NULL) {
				// insertion must be in node referenced by Parent_Ptr
					Parent_Ptr->Right_Ptr = NewNode_Ptr;
					Q_OKInsert = TRUE;
				}
				else {
				// node referenced by Parent_Ptr is not the last node
				// on the way to where NewNode_Ptr is to be inserted
					Parent_Ptr = Parent_Ptr->Right_Ptr;
				}
			}
		}
	}
}


// PRE	Q_BST_Present_R(Key, BST)
// POST	RETURNS record with key value Key from BST
Element_T
BST_Retrieve_R(Key_T Key, BST_T BST) {

	if (Key < Get_Key(Get_Root(BST))) {
		return(BST_Retrieve_R(Key, Get_Left(BST)));
	}
	else {
		if (Key > Get_Key(Get_Root(BST))) {
			return(BST_Retrieve_R(Key, Get_Right(BST)));
		}
		else {
			return(Get_Root(BST));
		}
	}
}


// PRE	NOT Q_EmptyTree(BST)
//	POST	BST' = BST without root node
void
BST_Delete_Root(BST_T &BST) {
	Node_Ptr_T	Root_Ptr,
					*RightMost_Ptr_Ptr,
					RightMost_Ptr;

	Root_Ptr = (Node_Ptr_T) BST;

	if (Root_Ptr->Left_Ptr == NULL) {
	// Node to be deleted has no left sub-tree
		BST = (BST_T) Root_Ptr->Right_Ptr;
		// Reset pointer in parent to right sub-tree
		// of node to be deleted
	}
	else {
		if (Root_Ptr->Right_Ptr == NULL) {
		// Node to be deleted has no right sub-tree
			BST = (BST_T) Root_Ptr->Left_Ptr;
			// Reset pointer in parent to left sub-tree
			// of node to be deleted
		}
		else {
		// Node to be deleted has left and right children
			RightMost_Ptr_Ptr = &(Root_Ptr->Left_Ptr);
			// RightMost_Ptr_Ptr pointing at the
			// pointer to the left sub-tree of the
			// node to be deleted
			for(;(*RightMost_Ptr_Ptr)->Right_Ptr != NULL;){
				  RightMost_Ptr_Ptr = &((*RightMost_Ptr_Ptr)->Right_Ptr);
			}
			// RightMost_Ptr_Ptr is pointing at the
			// pointer to the node whose right sub-tree
			// is empty - i.e. the rightmost node of
			// the left sub-tree of the node to be
			// deleted
			RightMost_Ptr = *RightMost_Ptr_Ptr;
			// RightMost_Ptr points at the rightmost node
			// of the left sub-tree of the note to be deleted
			*RightMost_Ptr_Ptr = (*RightMost_Ptr_Ptr)->Left_Ptr;
			// The pointer to the rightmost node is reset to
			// point to the left sub-tree of the rightmost node
			RightMost_Ptr->Left_Ptr = Root_Ptr->Left_Ptr;
			RightMost_Ptr->Right_Ptr = Root_Ptr->Right_Ptr;
			// The left and right sub-trees of RightMost_Ptr are
			//	now those of the node to be deleted
			BST = (BST_T) RightMost_Ptr;
			// RightMost_Ptr points at the new root so reset BST
		}
	}
	delete(Root_Ptr);
	// BST has been rearranged correctly
	// so now physically delete the node
}



// PRE	TRUE
// POST	IF (Q_BST_Present(Key, BST)
//				BST' is BST with record with Key deleted
void
BST_Delete(Key_T Key, BST_T &BST) {
	Node_Ptr_T	Parent_Ptr;
	BOOLEAN	Q_FoundDelete;

	if (BST != NULL) {
		if (Key == Get_Key(((Node_Ptr_T) BST)->Element)) {
			BST_Delete_Root(BST);
		}
		else {
			Q_FoundDelete = FALSE;
			Parent_Ptr = (Node_Ptr_T) BST;
			for(;!Q_FoundDelete;) {
				if (Key < Get_Key(Parent_Ptr->Element)) {
				// look ahead left
					if (Parent_Ptr->Left_Ptr == NULL) {
					// insertion must be in node referenced by Parent_Ptr
						Q_FoundDelete = TRUE;
					}
					else {
						if (Key == Get_Key(Parent_Ptr->Left_Ptr->Element)) {
							Q_FoundDelete = TRUE;
							BST_Delete_Root((BST_T) Parent_Ptr->Left_Ptr);
						}
						else {
						// node referenced by Parent_Ptr is not the last node
						// on the way to where NewNode_Ptr is to be inserted
							Parent_Ptr = Parent_Ptr->Left_Ptr;
						}
					}
				}
				else {
				// look ahead right
					if (Parent_Ptr->Right_Ptr == NULL) {
					// insertion must be in node referenced by Parent_Ptr
						Q_FoundDelete = TRUE;
					}
					else {
						if (Key == Get_Key(Parent_Ptr->Right_Ptr->Element)) {
							Q_FoundDelete = TRUE;
							BST_Delete_Root((BST_T) Parent_Ptr->Right_Ptr);
						}
						else {
						// node referenced by Parent_Ptr is not the last node
						// on the way to where NewNode_Ptr is to be inserted
							Parent_Ptr = Parent_Ptr->Right_Ptr;
						}
					}
				}
			}
		}
	}
}




