// Ken Brownsey
// 31/8/98
// bst_tree.cpp
// BST_T directly pointer based implementation

#include "my_const.h"
#include "bst_tree.h"

typedef struct TN_T {
	struct TN_T	*Left_Ptr,
					*Right_Ptr;
	Element_T	Element;
} Node_T, *Node_Ptr_T;

int	NumberAllocated = 0;


// PRE	TRUE
// POST	RETURNS the empty tree
BST_T
Create_Tree() {
	return(NULL);
}


// PRE	TRUE
// POST	if (Tree is empty RETURNS TRUE else RETURNS FALSE
BOOLEAN
Q_EmptyTree(BST_T Tree) {
	return(Tree == NULL);
}


// PRE  	Tree is not empty
// POST	RETURNS the root record
Element_T
Get_Root(BST_T Tree) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) Tree;

	return(Node_Ptr->Element);
}


// PRE	TRUE
// POST	if (Tree is not empty
//				RETURNS the left sub tree of Tree
//			else
//				RETURNS the empty tree
BST_T
Get_Left(BST_T Tree) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) Tree;

	if (Node_Ptr == NULL) {
		return(NULL);
	}
	else {
		return((BST_T) (Node_Ptr->Left_Ptr));
	}
}


// PRE	TRUE
// POST	if (Tree is not empty
//				RETURNS the right sub tree of Tree
//			else
//				RETURNS the empty tree
BST_T
Get_Right(BST_T Tree) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) Tree;

	if (Node_Ptr == NULL) {
		return(NULL);
	}
	else {
		return((BST_T)  Node_Ptr->Right_Ptr);
	}
}


// PRE	TRUE
// POST	RETURNS the tree whose root is Root,
//				left sub tree is Left,
//				right sub tree is Right
BST_T
Make_Tree(BST_T Left, Element_T Element, BST_T Right) {

	Node_Ptr_T	NewTree;

	NewTree = new Node_T;
	NumberAllocated++;
	NewTree->Element = Element;
	NewTree->Left_Ptr = (Node_Ptr_T) Left;
	NewTree->Right_Ptr = (Node_Ptr_T) Right;
	return((BST_T)  NewTree);
}


// PRE  - 	TRUE
// POST - 	RETURNS the number of nodes allocated
Get_NumberNodes_Allocated() {
	return(NumberAllocated);
}


// PRE	TRUE
// POST	if record with key Number is in BST
//				RETURNS TRUE
//			else
//				RETURNS FALSE
BOOLEAN
Q_BST_Present_R(Number_T Number, BST_T BST) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) BST;

	if (BST == NULL) {
		return(FALSE);
	}
	else {
		if (Number < Get_KAPRecNumber(Node_Ptr->Element)) {
			return(Q_BST_Present_R(Number, Node_Ptr->Left_Ptr));
		}
		else {
			if (Number > Get_KAPRecNumber(Get_Root(BST))) {
				return(Q_BST_Present_R(Number, Node_Ptr->Right_Ptr));
			}
			else {
				return(TRUE);
			}
		}
	}
}


// PRE	NOT Q_BST_Present_R(Rec, BST)
// POST	RETURNS BST with Rec inserted according to Q_Precedes
void
BST_Insert(BOOLEAN (* Q_Precedes) (Element_T, Element_T), Element_T Rec, BST_T &BST) {
	Node_Ptr_T	Node_Ptr,
					NewNode_Ptr,
					Parent;
	BOOLEAN	Q_OKInsert;

	Node_Ptr = (Node_Ptr_T) BST;
	NewNode_Ptr = new(Node_T);
	NewNode_Ptr->Left_Ptr 	= NULL;
	NewNode_Ptr->Element 	= Rec;
	NewNode_Ptr->Right_Ptr 	= NULL;
	if (Node_Ptr == NULL) {
		Node_Ptr = NewNode_Ptr;
	}
	else {
		Parent = Node_Ptr;
		Q_OKInsert = FALSE;
		for(;!Q_OKInsert;) {
			if (Q_Precedes(Rec, Parent->Element)) {
				if (Parent->Left_Ptr == NULL) {
					Parent->Left_Ptr = NewNode_Ptr;
					Q_OKInsert = TRUE;
				}
				else {
					Parent = Parent->Left_Ptr;
				}
			}
			else {
				if (Parent->Right_Ptr == NULL) {
					Parent->Right_Ptr = NewNode_Ptr;
					Q_OKInsert = TRUE;
				}
				else {
					Parent = Parent->Right_Ptr;
				}
			}
		}
	}
	BST = (BST_T)  Node_Ptr;
}


// PRE	Q_BST_Present_R(Number, BST)
// POST	RETURNS record with key value Number from BST
Element_T
BST_Retrieve_R(Number_T Number, BST_T BST) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) BST;

	if (Number < Get_KAPRecNumber(Node_Ptr->Element)) {
		return(BST_Retrieve_R(Number, Node_Ptr->Left_Ptr));
	}
	else {
		if (Number > Get_KAPRecNumber(Node_Ptr->Element)) {
			return(BST_Retrieve_R(Number, Node_Ptr->Right_Ptr));
		}
		else {
			return(Node_Ptr->Element);
		}
	}
}


// PRE	NOT Q_EmptyTree(BST)
// POST	RETURNS rightmost Element_T of BST
Element_T
Get_RightMost_R(	Node_Ptr_T	Node_Ptr) {

	if (Node_Ptr->Right_Ptr == NULL) {
		return(Node_Ptr->Element);
	}
	else {
		return(Get_RightMost_R(Node_Ptr->Right_Ptr));
	}
}


// PRE	TRUE
// POST	if (Q_BST_Present(Rec, BST)
//				RETURNS BST with Rec deleted
//			else
//				RETURNS BST
void
BST_Delete_R(Number_T Number, BST_T &BST) {
	Node_Ptr_T	Node_Ptr;
	Element_T NewRoot;

	Node_Ptr = (Node_Ptr_T) BST;

	if (Node_Ptr != NULL) {

		if (Number < Get_KAPRecNumber(Node_Ptr->Element)) {
			BST_Delete_R(Number, (BST_T)  Node_Ptr->Left_Ptr);
		}

		else {
			if (Number > Get_KAPRecNumber(Node_Ptr->Element)) {
				BST_Delete_R(Number, (BST_T)  Node_Ptr->Right_Ptr);
			}
			else {
				if (Node_Ptr->Left_Ptr == NULL) {
					BST = (BST_T)  Node_Ptr->Right_Ptr;
					delete(Node_Ptr);
				}
				else {
					if (Node_Ptr->Right_Ptr == NULL) {
						BST = (BST_T)  Node_Ptr->Left_Ptr;
						delete(Node_Ptr);
					}
					else {
						NewRoot = Get_RightMost_R(Node_Ptr->Left_Ptr);
						Node_Ptr->Element = NewRoot;
						BST_Delete_R(Get_KAPRecNumber(NewRoot), (BST_T)  Node_Ptr->Left_Ptr);
					}
				}
			}
		}
	}
}





