// Ken Brownsey
// 31/8/98
// iterator.cpp
// iterator for BST

#include "stack.h"
#include "iterator.h"
// This next line is required only for demonstration purposes
// to view state of iterator stack
#include "data.h"

// record structure for iterator
typedef struct {
		Stack_T Stack;
		SElement_T Current;
		int CurrentNumber;
		BOOLEAN	Q_Bound;
}	BST_ItRec_T, *BST_ItRec_Ptr_T;


// PRE	TRUE
// POST	Iterator is created
BST_Iterator_T
Create_Iterator() {
	BST_ItRec_Ptr_T ItRec;

	ItRec = new(BST_ItRec_T);
	ItRec->Stack = Create_Stack();
	ItRec->Q_Bound = FALSE;
	return(ItRec);
}

// PRE	Create_Iterator(Iterator) has been called
//			AND since last, if any, call to
//			Destroy_Iterator(Iterator)
// POST	Iterator is destroyed
void
Destroy_Iterator(BST_Iterator_T Iterator) {
	BST_ItRec_Ptr_T ItRec;

	ItRec = (BST_ItRec_Ptr_T) Iterator;
	for(;!Q_EmptyStack(ItRec->Stack);) {
		Pop(ItRec->Stack);
	}
	Free_Stack(ItRec->Stack);
}

// PRE	TRUE
// POST	IF Start_Iteration(Iterator, BST) has been executed
//				correctly for some value BST
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
Q_Bound_To_BST(BST_Iterator_T Iterator) {
	BST_ItRec_Ptr_T ItRec;

	ItRec = (BST_ItRec_Ptr_T) Iterator;
	return(ItRec->Q_Bound);
}


// PRE	Q_Bound_To_BST(Iterator)
// POST	IF iteration is at ordinally keyed first node
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_StartOfIteration(BST_Iterator_T Iterator) {
	BST_ItRec_Ptr_T ItRec;

	ItRec = (BST_ItRec_Ptr_T) Iterator;
	return(ItRec->CurrentNumber = 1);
}


// PRE	Q_Bound_To_BST(Iterator)
// POST	IF iteration is at ordinally keyed last node
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_EndOfIteration(BST_Iterator_T Iterator) {
	BST_ItRec_Ptr_T ItRec;

	ItRec = (BST_ItRec_Ptr_T) Iterator;
	return(	Q_EmptyTree(Get_Right(ItRec->Current)) &&
				Q_EmptyStack(ItRec->Stack));
}


// PRE	NOT Q_EmptyTree(BST)
// POST	Q_StartOfIteration(Iterator')
void
Start_Iteration(BST_Iterator_T &Iterator, BST_T BST) {
	BST_T NodeIndex;
	BST_ItRec_Ptr_T ItRec;

	ItRec = (BST_ItRec_Ptr_T) Iterator;
	for (;!Q_EmptyStack(ItRec->Stack);) {
		 Pop(ItRec->Stack);
	}
	NodeIndex = BST;
	for (;!Q_EmptyTree(Get_Left(NodeIndex));) {
		Push(NodeIndex, ItRec->Stack);
		NodeIndex = Get_Left(NodeIndex);
	}
	// NodeIndex is the leftmost node of BST
	ItRec->Current = NodeIndex;
	ItRec->CurrentNumber = 1;
	ItRec->Q_Bound = TRUE;
}


// PRE	Start_Iteration(&Iterator, BST) has been called
//			for some value BST
//			AND NOT Q_EndOfIteration(Iterator)
// POST	NOT Q_Iteration_Reset(Iterator') AND
//					current is next node in iteration
void
Iterate(BST_Iterator_T &Iterator) {
	BST_ItRec_Ptr_T ItRec;

	ItRec = (BST_ItRec_Ptr_T) Iterator;
	(ItRec->CurrentNumber)++;
	if (Q_EmptyTree(Get_Right(ItRec->Current))) {
		 ItRec->Current = Top(ItRec->Stack);
		 Pop(ItRec->Stack);
	}
	else {
		 ItRec->Current = Get_Right(ItRec->Current);
		 for (;!Q_EmptyTree(Get_Left(ItRec->Current));) {
			  Push(ItRec->Current, ItRec->Stack);
			  ItRec->Current = Get_Left(ItRec->Current);
		 }
	}
}


// PRE     Q_Iteration_Started(Iterator)
// RETURNS current record in iteration
Element_T
Get_Current(BST_Iterator_T Iterator) {
	BST_ItRec_Ptr_T ItRec;

	ItRec = (BST_ItRec_Ptr_T) Iterator;
	return(Get_Root(ItRec->Current));
}


// PRE	Q_Iteration_Started(Iterator)
// POST	RETURNS number of current record in iteration
int
Get_CurrentNumber(BST_Iterator_T Iterator) {
	BST_ItRec_Ptr_T ItRec;

	ItRec = (BST_ItRec_Ptr_T) Iterator;
	return(ItRec->CurrentNumber);
}




