// clubs.cpp
// 31/8/98
// Binary search tree application
// Ken Brownsey

#include "bst_util.h"
#include <iostream.h>
#include "rd_str.h"
#include "string.h"
#define ReqSize	20

typedef 	BST_T ClubRolls_T[NumClubs];
typedef  struct	{
	BST_T			MainRoll,
					NameRoll;
	ClubRolls_T   ClubRolls;
}	ClubsDB_T;

// Global list
ClubsDB_T	ClubsDB;


//	PRE	TRUE
// POST	IF Rec1 precedes Rec2 by key field
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_Number_Precedes(KAPRec_T KAPRec1, KAPRec_T KAPRec2) {
	return(Get_KAPRecNumber(KAPRec1) < Get_KAPRecNumber(KAPRec2));
}


//	PRE	TRUE
// POST	IF Rec1 precedes Rec2 by name field
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_Name_Precedes(KAPRec_T KAPRec1, KAPRec_T KAPRec2) {
	return(strcmp(Get_KAPRecName(KAPRec1), Get_KAPRecName(KAPRec2)) < 0);
}


// PRE	TRUE
// POST	user is prompted for record
//			if (record is not in DBase
//				it is added to DBase
//			} else
//				appropriate message
void
Add_New_Record(ClubsDB_T &ClubsDB) {

	DataRec_T	DataRec;
	Club_T		Club;
	KAPRec_T		KAPRec;

	cout	<< "\nType record in\n";
	Read_Rec(DataRec);
	KAPRec = Make_KAPRec(DataRec);
	if (!Q_BST_Present_R(Get_Number(DataRec), ClubsDB.MainRoll)) {
		BST_Insert(Q_Number_Precedes, KAPRec, ClubsDB.MainRoll);
		BST_Insert(Q_Name_Precedes, KAPRec, ClubsDB.NameRoll);
		for (Club = FirstClub; Club <= LastClub; Club++) {
			if (Q_Member_Of_Club(Club, Get_DataRec(KAPRec))) {
				BST_Insert(Q_Number_Precedes, KAPRec, ClubsDB.ClubRolls[Club]);
			}
		}
	}
	else {
		cout	<<	"\nRecord with key ";
		Write_Number(Get_Number(DataRec));
		cout	<< " is already in database\n";
	}
}




// PRE	TRUE
// POST if (!Q_EmptyTree) {
//			prints out size of tree
//		} else
//			empty tree message
void
Print_Size(BST_T DBase) {
	if (!Q_EmptyTree(DBase)) {
		cout	<< "\nSize is "
				<< Size_R(DBase)
				<< "\n";
	}
	else {
		cout	<< "\Database is empty\n";
	}
}




// PRE	TRUE
// POST	User is prompted for filename
//			IF this is a file of Element_T's
//				these are loaded into DBase
//				except for those already there
//				which are printed with appropriate message
void
Load_Clubs_From_File(ClubsDB_T &ClubsDB) {
	char	*FName;
	ifstream	InFile;
	BOOLEAN	Q_OKRead;
	KAPRec_T	NextRec;
	Club_T	Club;

	cout	<< "\nPlease type name of file to load from ==> ";
	FName = Read_String();
	InFile.open(FName, ios::in);
	if (InFile == NULL) {
		cout	<< "\n\nError - cannot open "
				<< FName
				<< "\n\n";
	}
	else {
		cout	<< "\nLoading records from "
				<< FName;
		for(;!InFile.eof();) {
			NextRec = Make_KAPRec(Read_Rec_From_File(InFile, Q_OKRead));
			if (Q_OKRead) {
				if (!InFile.eof()) {
					if (!Q_BST_Present_R(Get_KAPRecNumber(NextRec), ClubsDB.MainRoll)) {
						cout	<< "\nRecord ==> ";
						Write_Rec(Get_DataRec(NextRec));
						BST_Insert(Q_Number_Precedes, NextRec, ClubsDB.MainRoll);
						BST_Insert(Q_Name_Precedes, NextRec, ClubsDB.NameRoll);
						for (Club = FirstClub; Club <= LastClub; Club++) {
							if (Q_Member_Of_Club(Club, Get_DataRec(NextRec))) {
								BST_Insert(Q_Number_Precedes, NextRec, ClubsDB.ClubRolls[Club]);
							}
						}
					}
					else {
						cout	<< "\nRecord with key ";
						Write_Number(Get_KAPRecNumber(NextRec));
						cout	<< " is already in database";
					}
				}
			}
			else {
				cout	<<	"\nThere is a problem reading from file "
						<<	FName
						<<	"\n\n";
			}
		}
	}
}



// PRE	TRUE
// POST	if Tree is not empty
//				prints out elements inorderwise
//			} else
//				appropriate message
void
Do_InOrder(BST_T Tree) {
	if (!Q_EmptyTree(Tree)) {
		InOrder_R(Tree);
	}
}


//	PRE   TRUE
//	POST	Prints out membership by number
void
Print_Members_ByNumber(ClubsDB_T ClubsDB) {

	if (!Q_EmptyTree(ClubsDB.MainRoll)) {
		Do_InOrder(ClubsDB.MainRoll);
	}
	else {
		cout	<< "\nDatabase is empty\n";
	}

}


//	PRE   TRUE
//	POST	Prints out membership by name
void
Print_Members_ByName(ClubsDB_T ClubsDB) {

	if (!Q_EmptyTree(ClubsDB.NameRoll)) {
		Do_InOrder(ClubsDB.NameRoll);
	}
	else {
		cout	<< "\nDatabase is empty\n";
	}

}



// PRE	TRUE
//	POST	Prompts user for club
//			IF club has members
//				Prints out members inorderwise
//			ELSE
//				Message
void
Print_ClubMembership(ClubsDB_T ClubsDB) {
	Club_T Club;

	Club = Read_Club();

	if (Q_ValidClub(Club)) {

		if (Q_EmptyTree(ClubsDB.ClubRolls[Club])) {
			cout	<< "\nThere are no members in ";
			Write_Club(Club);
		}
		else {
			cout	<< "\nMembers of ";
			Write_Club(Club);
			cout	<< " are ";
			InOrder_R(ClubsDB.ClubRolls[Club]);
		}
	}
	else {
		cout	<< "\nNot a valid club ";
	}
}


// PRE	TRUE
// POST	traversal menu displayed
void
Traversal_Menu() {

	cout	<< "***   Traversal Options  	  *****************\n";
	cout	<< "***   (i)norder traversal  	  **************\n";
	cout	<< "***   inorder traversal by (n)ame     *******\n";
	cout	<< "***   (c)lub inorder traversal  	  ********\n";
	cout	<< "***   (t)raversal menu 	  *****************\n";
	cout	<< "***   e(x)it to main menu	  	****************\n";
}


// PRE	TRUE
// POST Goes into a prompt, read and execute cycle
//		for traversal options
//		until 'x' is the character read
void
Traversals(ClubsDB_T ClubsDB) {

	char 	Request;
	static int	RequestCount = 0;

	for (Request = 'm'; Request != 'x'; RequestCount++) {

		switch(Request) {

			case 'i' : Print_Members_ByNumber(ClubsDB);
				break;
			case 'n' : Print_Members_ByName(ClubsDB);
				break;
			case 'c'	: Print_ClubMembership(ClubsDB);
				break;
			case 'm' : Traversal_Menu();
				break;
			default   : cout	<< "\n m for menu \n";
		}
		cout	<< "\n\nNext traversal request ["
				<< RequestCount
				<< "] ==> ";
		cin	>> Request;
	}
}



// PRE	TRUE
// POST	menu displayed
void
Menu() {

	cout	<< "***   Main Menu            	  **************\n";
	cout	<< "***   (a)dd a new record  	  **************\n";
	cout	<< "***   print (s)ize of database **************\n";
	cout	<< "***   (l)oad database from file  ************\n";
	cout	<< "***   (t)raversal options  ******************\n";
	cout	<< "***   (i)terate options  ******************\n";
	cout	<< "***   this (m)enu	       	****************\n";
	cout	<< "***   e(x)it			        	****************\n";
}


// PRE	TRUE
// POST	DBase initialised
void
Initialise(ClubsDB_T ClubsDB) {
	Club_T Club;

	ClubsDB.MainRoll = Create_Tree();
	for (Club = FirstClub; Club < LastClub; Club++) {
		ClubsDB.ClubRolls[Club] = Create_Tree();
	}
}


// PRE	TRUE
// POST Goes into a prompt, read and execute cycle
//		until 'x' is the character read
void
Loop(ClubsDB_T &ClubsDB) {

	char 	Request;
	int	RequestCount;

	for (Request = 'm', RequestCount = 0; Request != 'x'; RequestCount++) {

		switch(Request) {

			case 'a': Add_New_Record(ClubsDB);
				break;
			case 's': Print_Size(ClubsDB.MainRoll);
				break;
			case 'l': Load_Clubs_From_File(ClubsDB);
				break;
			case 't' : Traversals(ClubsDB);
				break;
			case 'm' : Menu();
				break;
			default   : cout	<< "\n m for menu \n";
		}
		cout	<< "\n\nNext request ["
				<< RequestCount
				<< "] ==> ";
		cin	>> Request;
	}
}


// PRE	TRUE
// POST	Clears up
void
Finalise(ClubsDB_T ClubsDB) {
	cout	<< "That is it !";
}


void
main () {
	Initialise(ClubsDB);
	Loop(ClubsDB);
	Finalise(ClubsDB);
}